# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class FirstPaymentDate(BaseModel):
    """First payment date

    :param due: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31., defaults to None
    :type due: str, optional
    """

    def __init__(self, due: str = None):
        if due is not None:
            self.due = due



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class BankAccountUpdated(BaseModel):
    """BankAccountUpdated

    :param id_: id_
    :type id_: str
    :param status: status
    :type status: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param updated_at: Long date-time format following ISO-8601
    :type updated_at: str
    """

    def __init__(self, id_: str, status: str, created_at: str, updated_at: str):
        self.id_ = id_
        self.status = status
        self.created_at = created_at
        self.updated_at = updated_at



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class CostQuote(BaseModel):
    """CostQuote

    :param name: Cost's name, defaults to None
    :type name: str, optional
    :param currency: Currency code., defaults to None
    :type currency: str, optional
    :param total: Total cost in local currency, defaults to None
    :type total: float, optional
    :param total_usd: Total cost in USD, defaults to None
    :type total_usd: float, optional
    """

    def __init__(
        self,
        name: str = None,
        currency: str = None,
        total: float = None,
        total_usd: float = None,
    ):
        if name is not None:
            self.name = name
        if currency is not None:
            self.currency = self._pattern_matching(currency, "^[A-Z]{3}$", "currency")
        if total is not None:
            self.total = total
        if total_usd is not None:
            self.total_usd = total_usd



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class UpdateWorkerDepartment(BaseModel):
    """UpdateWorkerDepartment

    :param department_id: Unique identifier of this resource.
    :type department_id: str
    """

    def __init__(self, department_id: str):
        self.department_id = department_id



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .bank_account_status import BankAccountStatus


@JsonMap({"id_": "id"})
class BankAccountAdded(BaseModel):
    """BankAccountAdded

    :param id_: ID of Bank Account., defaults to None
    :type id_: str, optional
    :param status: Bank Account status, defaults to None
    :type status: BankAccountStatus, optional
    :param created_at: Long date-time format following ISO-8601, defaults to None
    :type created_at: str, optional
    :param updated_at: Long date-time format following ISO-8601, defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        status: BankAccountStatus = None,
        created_at: str = None,
        updated_at: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if status is not None:
            self.status = self._enum_matching(
                status, BankAccountStatus.list(), "status"
            )
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .gp_contract_salary_scale_enum import GpContractSalaryScaleEnum


@JsonMap({})
class GpEmployeeCompensationUpdateData(BaseModel):
    """GpEmployeeCompensationUpdateData

    :param scale: The scale to which a given salary applies.
    :type scale: GpContractSalaryScaleEnum
    :param salary: Compensation salary.
    :type salary: float
    :param effective_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type effective_date: str
    """

    def __init__(
        self, scale: GpContractSalaryScaleEnum, salary: float, effective_date: str
    ):
        self.scale = self._enum_matching(
            scale, GpContractSalaryScaleEnum.list(), "scale"
        )
        self.salary = salary
        self.effective_date = effective_date



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class FileRefTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar APPLICATION_PDF: "application/pdf"
    :vartype APPLICATION_PDF: str
    :cvar TEXT_CSV: "text/csv"
    :vartype TEXT_CSV: str
    :cvar TEXT_PLAIN: "text/plain"
    :vartype TEXT_PLAIN: str
    :cvar IMAGE_JPEG: "image/jpeg"
    :vartype IMAGE_JPEG: str
    :cvar IMAGE_PNG: "image/png"
    :vartype IMAGE_PNG: str
    """

    APPLICATION_PDF = "application/pdf"
    TEXT_CSV = "text/csv"
    TEXT_PLAIN = "text/plain"
    IMAGE_JPEG = "image/jpeg"
    IMAGE_PNG = "image/png"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, FileRefTypeEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_to_create_pay_as_you_go_time_based import (
    ContractToCreatePayAsYouGoTimeBased,
)


@JsonMap({})
class ContractToCreateContainerPayAsYouGoTimeBased(BaseModel):
    """ContractToCreateContainerPayAsYouGoTimeBased

    :param data: data
    :type data: ContractToCreatePayAsYouGoTimeBased
    """

    def __init__(self, data: ContractToCreatePayAsYouGoTimeBased):
        self.data = self._define_object(data, ContractToCreatePayAsYouGoTimeBased)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class PaymentStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar PAID: "paid"
    :vartype PAID: str
    :cvar PROCESSING: "processing"
    :vartype PROCESSING: str
    """

    PAID = "paid"
    PROCESSING = "processing"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PaymentStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_reviews_to_create import InvoiceAdjustmentReviewsToCreate


@JsonMap({})
class InvoiceAdjustmentReviewsToCreateContainer(BaseModel):
    """InvoiceAdjustmentReviewsToCreateContainer

    :param data: data
    :type data: InvoiceAdjustmentReviewsToCreate
    """

    def __init__(self, data: InvoiceAdjustmentReviewsToCreate):
        self.data = self._define_object(data, InvoiceAdjustmentReviewsToCreate)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class RequesterTimeOff(BaseModel):
    """RequesterTimeOff

    :param is_part_time: is_part_time, defaults to None
    :type is_part_time: bool, optional
    :param has_reached_end_date: has_reached_end_date, defaults to None
    :type has_reached_end_date: bool, optional
    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param uuid: uuid, defaults to None
    :type uuid: str, optional
    :param hris_profile_id: hris_profile_id, defaults to None
    :type hris_profile_id: str, optional
    :param client_profile_id: client_profile_id, defaults to None
    :type client_profile_id: str, optional
    :param organization_id: organization_id, defaults to None
    :type organization_id: float, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param worker_type: worker_type, defaults to None
    :type worker_type: str, optional
    :param end_date: end_date, defaults to None
    :type end_date: str, optional
    :param part_time_percentage: part_time_percentage, defaults to None
    :type part_time_percentage: str, optional
    :param created_at: created_at, defaults to None
    :type created_at: str, optional
    :param updated_at: updated_at, defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        is_part_time: bool = None,
        has_reached_end_date: bool = None,
        id_: str = None,
        uuid: str = None,
        hris_profile_id: str = None,
        client_profile_id: str = None,
        organization_id: float = None,
        country: str = None,
        worker_type: str = None,
        end_date: str = None,
        part_time_percentage: str = None,
        created_at: str = None,
        updated_at: str = None,
    ):
        if is_part_time is not None:
            self.is_part_time = is_part_time
        if has_reached_end_date is not None:
            self.has_reached_end_date = has_reached_end_date
        if id_ is not None:
            self.id_ = id_
        if uuid is not None:
            self.uuid = uuid
        if hris_profile_id is not None:
            self.hris_profile_id = hris_profile_id
        if client_profile_id is not None:
            self.client_profile_id = client_profile_id
        if organization_id is not None:
            self.organization_id = organization_id
        if country is not None:
            self.country = country
        if worker_type is not None:
            self.worker_type = worker_type
        if end_date is not None:
            self.end_date = end_date
        if part_time_percentage is not None:
            self.part_time_percentage = part_time_percentage
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class OutputToCreateFileRef(BaseModel):
    """OutputToCreateFileRef

    :param key: key
    :type key: str
    :param url: url
    :type url: str
    """

    def __init__(self, key: str, url: str):
        self.key = key
        self.url = url



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .profile_type_enum import ProfileTypeEnum


@JsonMap({"id_": "id"})
class TimeoffsProfile(BaseModel):
    """TimeoffsProfile

    :param id_: id_
    :type id_: IdentifierValue
    :param name: name
    :type name: str
    :param pic_url: pic_url, defaults to None
    :type pic_url: str, optional
    :param profile_type: profile_type
    :type profile_type: ProfileTypeEnum
    :param is_employee: Only valid for requester. True if requester is an employee., defaults to None
    :type is_employee: bool, optional
    """

    def __init__(
        self,
        id_: IdentifierValue,
        name: str,
        profile_type: ProfileTypeEnum,
        pic_url: str = None,
        is_employee: bool = None,
    ):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.name = name
        if pic_url is not None:
            self.pic_url = pic_url
        self.profile_type = self._enum_matching(
            profile_type, ProfileTypeEnum.list(), "profile_type"
        )
        if is_employee is not None:
            self.is_employee = is_employee



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class SignaturesOfBasicContract(BaseModel):
    """SignaturesOfBasicContract

    :param client_signed_at: Long date-time format following ISO-8601
    :type client_signed_at: str
    :param worker_signature: Worker's signature.
    :type worker_signature: str
    :param worker_signed_at: Long date-time format following ISO-8601
    :type worker_signed_at: str
    """

    def __init__(
        self, client_signed_at: str, worker_signature: str, worker_signed_at: str
    ):
        self.client_signed_at = client_signed_at
        self.worker_signature = worker_signature
        self.worker_signed_at = worker_signed_at



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class LastCycle(BaseModel):
    """The last payment cycle.

    :param start: Long date-time format following ISO-8601, defaults to None
    :type start: str, optional
    :param end: Long date-time format following ISO-8601, defaults to None
    :type end: str, optional
    :param completion_date: Long date-time format following ISO-8601, defaults to None
    :type completion_date: str, optional
    """

    def __init__(self, start: str = None, end: str = None, completion_date: str = None):
        if start is not None:
            self.start = start
        if end is not None:
            self.end = end
        if completion_date is not None:
            self.completion_date = completion_date


class FinalPaymentCalculatedCalculationType(Enum):
    """An enumeration representing different categories.

    :cvar CUSTOM_AMOUNT: "CUSTOM_AMOUNT"
    :vartype CUSTOM_AMOUNT: str
    :cvar WORK_DAYS: "WORK_DAYS"
    :vartype WORK_DAYS: str
    :cvar CALENDAR_DAYS: "CALENDAR_DAYS"
    :vartype CALENDAR_DAYS: str
    :cvar FULL_AMOUNT: "FULL_AMOUNT"
    :vartype FULL_AMOUNT: str
    """

    CUSTOM_AMOUNT = "CUSTOM_AMOUNT"
    WORK_DAYS = "WORK_DAYS"
    CALENDAR_DAYS = "CALENDAR_DAYS"
    FULL_AMOUNT = "FULL_AMOUNT"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                FinalPaymentCalculatedCalculationType._member_map_.values(),
            )
        )


@JsonMap({})
class FinalPaymentCalculated(BaseModel):
    """FinalPaymentCalculated

    :param daily_rate: The daily rate paid.
    :type daily_rate: str
    :param total: Total amount to pay.
    :type total: str
    :param remaining_days: Dais remaining to end the contract.
    :type remaining_days: str
    :param last_cycle: The last payment cycle.
    :type last_cycle: LastCycle
    :param calculation_type: Either works days or calendar days
    :type calculation_type: FinalPaymentCalculatedCalculationType
    :param workweek_start: The day the work week starts, 0 to 6
    :type workweek_start: float
    :param workweek_end: The day the work week starts, 0 to 6
    :type workweek_end: float
    """

    def __init__(
        self,
        daily_rate: str,
        total: str,
        remaining_days: str,
        last_cycle: LastCycle,
        calculation_type: FinalPaymentCalculatedCalculationType,
        workweek_start: float,
        workweek_end: float,
    ):
        self.daily_rate = daily_rate
        self.total = total
        self.remaining_days = remaining_days
        self.last_cycle = self._define_object(last_cycle, LastCycle)
        self.calculation_type = self._enum_matching(
            calculation_type,
            FinalPaymentCalculatedCalculationType.list(),
            "calculation_type",
        )
        self.workweek_start = workweek_start
        self.workweek_end = workweek_end



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class InvoiceAdjustmentToUpdate(BaseModel):
    """Details of invoice adjustment to update.

    :param description: description
    :type description: str
    :param amount: amount, defaults to None
    :type amount: float, optional
    """

    def __init__(self, description: str, amount: float = None):
        self.description = description
        if amount is not None:
            self.amount = amount



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.payment_list_container import PaymentListContainer
from ..models.payment_break_down_container import PaymentBreakDownContainer
from ..models.invoice_list_container import InvoiceListContainer
from ..models.invoice_download_link_container import InvoiceDownloadLinkContainer
from ..models.get_payment_list_entities import GetPaymentListEntities
from ..models.get_payment_list_currencies import GetPaymentListCurrencies
from ..models.get_invoice_list_entities import GetInvoiceListEntities
from ..models.deel_invoice_list_container import DeelInvoiceListContainer


class AccountingService(BaseService):

    @cast_models
    def get_invoice_list(
        self,
        issued_from_date: str = None,
        issued_to_date: str = None,
        entities: GetInvoiceListEntities = None,
        limit: float = None,
        offset: float = None,
    ) -> InvoiceListContainer:
        """Retrieve a list of paid invoices for your workforce.

        :param issued_from_date: to get records created after given issue date, defaults to None
        :type issued_from_date: str, optional
        :param issued_to_date: to get records created before given issued date, defaults to None
        :type issued_to_date: str, optional
        :param entities: Filter by legal entity type; company, or individual, defaults to None
        :type entities: GetInvoiceListEntities, optional
        :param limit: Return a page of results with given number of records; NOTE technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param offset: Return a page of results after given index of row; NOTE technically ALL query parameters are strings or array of strings', defaults to None
        :type offset: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceListContainer
        """

        Validator(str).is_optional().validate(issued_from_date)
        Validator(str).is_optional().validate(issued_to_date)
        Validator(GetInvoiceListEntities).is_optional().validate(entities)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(float).is_optional().min(0).max(999999999).validate(offset)

        serialized_request = (
            Serializer(f"{self.base_url}/invoices", self.get_default_headers())
            .add_query("issued_from_date", issued_from_date)
            .add_query("issued_to_date", issued_to_date)
            .add_query("entities", entities)
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return InvoiceListContainer._unmap(response)

    @cast_models
    def get_deel_invoice_list(
        self, contract_id: str, limit: float = None, offset: float = None
    ) -> DeelInvoiceListContainer:
        """Retrieve a list of invoices related to Deel fees.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param limit: Return a page of results with given number of records; NOTE technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param offset: Return a page of results after given index of row; NOTE technically ALL query parameters are strings or array of strings', defaults to None
        :type offset: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: DeelInvoiceListContainer
        """

        Validator(str).validate(contract_id)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(float).is_optional().min(0).max(999999999).validate(offset)

        serialized_request = (
            Serializer(f"{self.base_url}/invoices/deel", self.get_default_headers())
            .add_query("limit", limit)
            .add_query("offset", offset)
            .add_query("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return DeelInvoiceListContainer._unmap(response)

    @cast_models
    def get_billing_invoice_download_link(
        self, invoice_id: str
    ) -> InvoiceDownloadLinkContainer:
        """Get link to download the invoice PDF.

        :param invoice_id: ID of an existing invoice
        :type invoice_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceDownloadLinkContainer
        """

        Validator(str).validate(invoice_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoices/{{invoice_id}}/download",
                self.get_default_headers(),
            )
            .add_path("invoice_id", invoice_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return InvoiceDownloadLinkContainer._unmap(response)

    @cast_models
    def get_payment_list(
        self,
        date_from: str = None,
        date_to: str = None,
        currencies: GetPaymentListCurrencies = None,
        entities: GetPaymentListEntities = None,
    ) -> PaymentListContainer:
        """Retrieve a list of payments made to Deel.

        :param date_from: Filtered results will include records created on or after the provided date., defaults to None
        :type date_from: str, optional
        :param date_to: Filtered results will include records created before the provided date., defaults to None
        :type date_to: str, optional
        :param currencies: Currency codes of contracts to filter., defaults to None
        :type currencies: GetPaymentListCurrencies, optional
        :param entities: Filter by legal entity type; company, or individual, defaults to None
        :type entities: GetPaymentListEntities, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PaymentListContainer
        """

        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_from)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_to)
        Validator(GetPaymentListCurrencies).is_optional().validate(currencies)
        Validator(GetPaymentListEntities).is_optional().validate(entities)

        serialized_request = (
            Serializer(f"{self.base_url}/payments", self.get_default_headers())
            .add_query("date_from", date_from, nullable=True)
            .add_query("date_to", date_to, nullable=True)
            .add_query("currencies", currencies)
            .add_query("entities", entities)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return PaymentListContainer._unmap(response)

    @cast_models
    def get_payments_break_down_by_id(
        self, payment_id: str
    ) -> PaymentBreakDownContainer:
        """Get a full breakdown of a payment made to Deel. Breakdown will include individual invoices and Deel fee as line items.

        :param payment_id: payment_id
        :type payment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PaymentBreakDownContainer
        """

        Validator(str).validate(payment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/payments/{{payment_id}}/breakdown",
                self.get_default_headers(),
            )
            .add_path("payment_id", payment_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return PaymentBreakDownContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Entitlement(BaseModel):
    """Entitlement

    :param accrual_amount: Accrual amount of the time off policy., defaults to None
    :type accrual_amount: str, optional
    :param accrual_unit: Accrual unit of the time off policy., defaults to None
    :type accrual_unit: str, optional
    :param total: Total time off policy., defaults to None
    :type total: str, optional
    :param unit_amount: Unit amount of the time off policy., defaults to None
    :type unit_amount: float, optional
    :param unit: Unit of the time off policy., defaults to None
    :type unit: str, optional
    :param accrued: Accrued time off policy., defaults to None
    :type accrued: str, optional
    """

    def __init__(
        self,
        accrual_amount: str = None,
        accrual_unit: str = None,
        total: str = None,
        unit_amount: float = None,
        unit: str = None,
        accrued: str = None,
    ):
        if accrual_amount is not None:
            self.accrual_amount = accrual_amount
        if accrual_unit is not None:
            self.accrual_unit = accrual_unit
        if total is not None:
            self.total = total
        if unit_amount is not None:
            self.unit_amount = unit_amount
        if unit is not None:
            self.unit = unit
        if accrued is not None:
            self.accrued = accrued


@JsonMap({"id_": "id", "type_": "type"})
class EmployeeTimeoffsPoliciesItem(BaseModel):
    """EmployeeTimeoffsPoliciesItem

    :param id_: Unique identifier for the time off policy.
    :type id_: str
    :param type_: Type of time off policy.
    :type type_: str
    :param name: Name of the time off policy.
    :type name: str
    :param description: Description of the time off policy.
    :type description: str
    :param unit_amount: Unit amount of the time off policy.
    :type unit_amount: float
    :param used: Used time off policy.
    :type used: float
    :param tracking_start_date: Start date of the time off policy.
    :type tracking_start_date: str
    :param tracking_end_date: End date of the time off policy.
    :type tracking_end_date: any
    :param entitlement: entitlement
    :type entitlement: Entitlement
    """

    def __init__(
        self,
        id_: str,
        type_: str,
        name: str,
        description: str,
        unit_amount: float,
        used: float,
        tracking_start_date: str,
        tracking_end_date: any,
        entitlement: Entitlement,
    ):
        self.id_ = id_
        self.type_ = type_
        self.name = name
        self.description = description
        self.unit_amount = unit_amount
        self.used = used
        self.tracking_start_date = tracking_start_date
        self.tracking_end_date = tracking_end_date
        self.entitlement = self._define_object(entitlement, Entitlement)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class HiringStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar ACTIVE: "active"
    :vartype ACTIVE: str
    :cvar INACTIVE: "inactive"
    :vartype INACTIVE: str
    :cvar ONBOARDING: "onboarding"
    :vartype ONBOARDING: str
    """

    ACTIVE = "active"
    INACTIVE = "inactive"
    ONBOARDING = "onboarding"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, HiringStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_termination_result import ContractTerminationResult


@JsonMap({})
class ContractTerminationResultContainer(BaseModel):
    """ContractTerminationResultContainer

    :param data: data
    :type data: ContractTerminationResult
    """

    def __init__(self, data: ContractTerminationResult):
        self.data = self._define_object(data, ContractTerminationResult)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class WeekDaysEnum(Enum):
    """An enumeration representing different categories.

    :cvar SUNDAY: "Sunday"
    :vartype SUNDAY: str
    :cvar MONDAY: "Monday"
    :vartype MONDAY: str
    :cvar TUESDAY: "Tuesday"
    :vartype TUESDAY: str
    :cvar WEDNESDAY: "Wednesday"
    :vartype WEDNESDAY: str
    :cvar THURSDAY: "Thursday"
    :vartype THURSDAY: str
    :cvar FRIDAY: "Friday"
    :vartype FRIDAY: str
    :cvar SATURDAY: "Saturday"
    :vartype SATURDAY: str
    """

    SUNDAY = "Sunday"
    MONDAY = "Monday"
    TUESDAY = "Tuesday"
    WEDNESDAY = "Wednesday"
    THURSDAY = "Thursday"
    FRIDAY = "Friday"
    SATURDAY = "Saturday"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, WeekDaysEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class CompensationDetailsOfContract(BaseModel):
    """CompensationDetailsOfContract

    :param currency_code: currency_code
    :type currency_code: str
    :param amount: amount
    :type amount: str
    :param scale: scale
    :type scale: str
    :param frequency: frequency
    :type frequency: str
    :param first_payment: first_payment
    :type first_payment: str
    :param first_payment_date: Long date-time format following ISO-8601
    :type first_payment_date: str
    :param gross_annual_salary: gross_annual_salary
    :type gross_annual_salary: str
    :param gross_signing_bonus: gross_signing_bonus
    :type gross_signing_bonus: str
    :param gross_variable_bonus: gross_variable_bonus
    :type gross_variable_bonus: str
    """

    def __init__(
        self,
        currency_code: str,
        amount: str,
        scale: str,
        frequency: str,
        first_payment: str,
        first_payment_date: str,
        gross_annual_salary: str,
        gross_signing_bonus: str,
        gross_variable_bonus: str,
    ):
        self.currency_code = self._pattern_matching(
            currency_code, "^[A-Z]{3}$", "currency_code"
        )
        self.amount = amount
        self.scale = scale
        self.frequency = frequency
        self.first_payment = first_payment
        self.first_payment_date = first_payment_date
        self.gross_annual_salary = gross_annual_salary
        self.gross_signing_bonus = gross_signing_bonus
        self.gross_variable_bonus = gross_variable_bonus



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class InvoiceAdjustmentTypeToCreateEnum(Enum):
    """An enumeration representing different categories.

    :cvar BONUS: "bonus"
    :vartype BONUS: str
    :cvar COMMISSION: "commission"
    :vartype COMMISSION: str
    :cvar DEDUCTION: "deduction"
    :vartype DEDUCTION: str
    :cvar EXPENSE: "expense"
    :vartype EXPENSE: str
    :cvar OTHER: "other"
    :vartype OTHER: str
    :cvar OVERTIME: "overtime"
    :vartype OVERTIME: str
    :cvar TIME_OFF: "time_off"
    :vartype TIME_OFF: str
    :cvar VAT: "vat"
    :vartype VAT: str
    """

    BONUS = "bonus"
    COMMISSION = "commission"
    DEDUCTION = "deduction"
    EXPENSE = "expense"
    OTHER = "other"
    OVERTIME = "overtime"
    TIME_OFF = "time_off"
    VAT = "vat"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                InvoiceAdjustmentTypeToCreateEnum._member_map_.values(),
            )
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .invoice_adjustment_status_enum import InvoiceAdjustmentStatusEnum


@JsonMap({"id_": "id"})
class InvoiceAdjustmentCreated(BaseModel):
    """Details of invoice adjustment created.

    :param created: created
    :type created: bool
    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param status: status of invoice adjustment, defaults to None
    :type status: InvoiceAdjustmentStatusEnum, optional
    :param created_at: Long date-time format following ISO-8601, defaults to None
    :type created_at: str, optional
    """

    def __init__(
        self,
        created: bool,
        id_: IdentifierValue = None,
        status: InvoiceAdjustmentStatusEnum = None,
        created_at: str = None,
    ):
        self.created = created
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if status is not None:
            self.status = self._enum_matching(
                status, InvoiceAdjustmentStatusEnum.list(), "status"
            )
        if created_at is not None:
            self.created_at = created_at



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class AdjustmentCategory(BaseModel):
    """AdjustmentCategory

    :param id_: Unique identifier of an adjustment category., defaults to None
    :type id_: str, optional
    :param unit_type: The unit type of the adjustment category., defaults to None
    :type unit_type: str, optional
    :param label: The label of the adjustment category., defaults to None
    :type label: str, optional
    :param name: The name of the adjustment category., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        unit_type: str = None,
        label: str = None,
        name: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if unit_type is not None:
            self.unit_type = unit_type
        if label is not None:
            self.label = label
        if name is not None:
            self.name = name



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class WorkStatementCycleEndTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar DAY_OF_WEEK: "DAY_OF_WEEK"
    :vartype DAY_OF_WEEK: str
    :cvar DAY_OF_LAST_WEEK: "DAY_OF_LAST_WEEK"
    :vartype DAY_OF_LAST_WEEK: str
    :cvar DAY_OF_MONTH: "DAY_OF_MONTH"
    :vartype DAY_OF_MONTH: str
    """

    DAY_OF_WEEK = "DAY_OF_WEEK"
    DAY_OF_LAST_WEEK = "DAY_OF_LAST_WEEK"
    DAY_OF_MONTH = "DAY_OF_MONTH"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, WorkStatementCycleEndTypeEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .off_cycle_payment import OffCyclePayment


@JsonMap({})
class OffCyclePaymentContainer(BaseModel):
    """OffCyclePaymentContainer

    :param data: data
    :type data: OffCyclePayment
    """

    def __init__(self, data: OffCyclePayment):
        self.data = self._define_object(data, OffCyclePayment)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class PageInfoWithoutCursorNew(BaseModel):
    """PageInfoWithoutCursorNew

    :param total_rows: total_rows
    :type total_rows: float
    :param items_per_page: items_per_page
    :type items_per_page: float
    :param offset: offset
    :type offset: float
    """

    def __init__(self, total_rows: float, items_per_page: float, offset: float):
        self.total_rows = total_rows
        self.items_per_page = items_per_page
        self.offset = offset



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .employee_timeoffs_policies_item import EmployeeTimeoffsPoliciesItem


@JsonMap({})
class EmployeeTimeoffsPoliciesContainer(BaseModel):
    """EmployeeTimeoffsPoliciesContainer

    :param data: List of employee time offs policies.
    :type data: List[EmployeeTimeoffsPoliciesItem]
    """

    def __init__(self, data: List[EmployeeTimeoffsPoliciesItem]):
        self.data = self._define_list(data, EmployeeTimeoffsPoliciesItem)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .candidate_status_enum import CandidateStatusEnum


@JsonMap({})
class CandidateToPatch(BaseModel):
    """Details of candidate to update

    :param first_name: Candidate's first name., defaults to None
    :type first_name: str, optional
    :param last_name: Candidate's last name., defaults to None
    :type last_name: str, optional
    :param status: Offer status
    :type status: CandidateStatusEnum
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31., defaults to None
    :type start_date: str, optional
    :param job_title: Job Title., defaults to None
    :type job_title: str, optional
    :param link: Link to candidate's profile in ATS., defaults to None
    :type link: str, optional
    :param email: Worker's email address., defaults to None
    :type email: str, optional
    :param nationality: nationality, defaults to None
    :type nationality: str, optional
    :param country: Country code., defaults to None
    :type country: str, optional
    :param state: Job location state., defaults to None
    :type state: str, optional
    """

    def __init__(
        self,
        status: CandidateStatusEnum,
        first_name: str = None,
        last_name: str = None,
        start_date: str = None,
        job_title: str = None,
        link: str = None,
        email: str = None,
        nationality: str = None,
        country: str = None,
        state: str = None,
    ):
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        self.status = self._enum_matching(status, CandidateStatusEnum.list(), "status")
        if start_date is not None:
            self.start_date = start_date
        if job_title is not None:
            self.job_title = job_title
        if link is not None:
            self.link = link
        if email is not None:
            self.email = email
        if nationality is not None:
            self.nationality = nationality
        if country is not None:
            self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")
        if state is not None:
            self.state = state



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .job_title import JobTitle


@JsonMap({})
class Page(BaseModel):
    """Page

    :param cursor: Use for pagination to get next set of records after the given cursor.
    :type cursor: str
    """

    def __init__(self, cursor: str):
        self.cursor = cursor


@JsonMap({})
class JobTitleListContainer(BaseModel):
    """JobTitleListContainer

    :param data: data
    :type data: List[JobTitle]
    :param page: page
    :type page: Page
    """

    def __init__(self, data: List[JobTitle], page: Page):
        self.data = self._define_list(data, JobTitle)
        self.page = self._define_object(page, Page)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GetEmployeeComplianceDocumentTemplateDownloadLinkContainerData(BaseModel):
    """GetEmployeeComplianceDocumentTemplateDownloadLinkContainerData

    :param url: The URL for the requested compliance document template., defaults to None
    :type url: str, optional
    :param expires_at: Expiration date time of download URL., defaults to None
    :type expires_at: str, optional
    """

    def __init__(self, url: str = None, expires_at: str = None):
        if url is not None:
            self.url = url
        if expires_at is not None:
            self.expires_at = expires_at


@JsonMap({})
class GetEmployeeComplianceDocumentTemplateDownloadLinkContainer(BaseModel):
    """GetEmployeeComplianceDocumentTemplateDownloadLinkContainer

    :param data: data
    :type data: GetEmployeeComplianceDocumentTemplateDownloadLinkContainerData
    """

    def __init__(
        self, data: GetEmployeeComplianceDocumentTemplateDownloadLinkContainerData
    ):
        self.data = self._define_object(
            data, GetEmployeeComplianceDocumentTemplateDownloadLinkContainerData
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_type_enum import ContractTypeEnum
from .contract_status_enum import ContractStatusEnum
from .seniority import Seniority
from .contract_who_reports_enum import ContractWhoReportsEnum
from .client_of_contract import ClientOfContract
from .worker_of_contract import WorkerOfContract
from .invitations_of_basic_contract import InvitationsOfBasicContract
from .signatures_of_contract import SignaturesOfContract
from .compensation_details_of_contract import CompensationDetailsOfContract
from .employment_details_of_contract import EmploymentDetailsOfContract
from .contract_template import ContractTemplate
from .contract_custom_field import ContractCustomField
from .eor_quote_base import EorQuoteBase


@JsonMap({"id_": "id", "type_": "type"})
class Contract(BaseModel):
    """Contract

    :param id_: id_
    :type id_: str
    :param title: title
    :type title: str
    :param type_: Type of a contract
    :type type_: ContractTypeEnum
    :param status: Status of a contract in Deel workflow
    :type status: ContractStatusEnum
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param job_title: job_title, defaults to None
    :type job_title: str, optional
    :param seniority: Seniority describes level of expertise at a job e.g. junior
    :type seniority: Seniority
    :param who_reports: Flag to indicate who is supposed to provide regular reports, defaults to None
    :type who_reports: ContractWhoReportsEnum, optional
    :param scope_of_work: scope_of_work, defaults to None
    :type scope_of_work: str, optional
    :param start_date: Long date-time format following ISO-8601
    :type start_date: str
    :param termination_date: Long date-time format following ISO-8601
    :type termination_date: str
    :param special_clause: special_clause
    :type special_clause: str
    :param is_archived: is_archived
    :type is_archived: bool
    :param client: client
    :type client: ClientOfContract
    :param worker: worker
    :type worker: WorkerOfContract
    :param invitations: invitations
    :type invitations: InvitationsOfBasicContract
    :param signatures: signatures
    :type signatures: SignaturesOfContract
    :param compensation_details: compensation_details
    :type compensation_details: CompensationDetailsOfContract
    :param employment_details: employment_details
    :type employment_details: EmploymentDetailsOfContract
    :param notice_period: Days before to notice the termination of contract for eather party., defaults to None
    :type notice_period: float, optional
    :param contract_template: Contract template details., defaults to None
    :type contract_template: ContractTemplate, optional
    :param custom_fields: custom_fields, defaults to None
    :type custom_fields: List[ContractCustomField], optional
    :param quote: EOR quote approved by Deel, defaults to None
    :type quote: EorQuoteBase, optional
    :param external_id: A unique identifier for the object provided by an external system., defaults to None
    :type external_id: str, optional
    """

    def __init__(
        self,
        id_: str,
        title: str,
        type_: ContractTypeEnum,
        status: ContractStatusEnum,
        created_at: str,
        seniority: Seniority,
        start_date: str,
        termination_date: str,
        special_clause: str,
        is_archived: bool,
        client: ClientOfContract,
        worker: WorkerOfContract,
        invitations: InvitationsOfBasicContract,
        signatures: SignaturesOfContract,
        compensation_details: CompensationDetailsOfContract,
        employment_details: EmploymentDetailsOfContract,
        job_title: str = None,
        who_reports: ContractWhoReportsEnum = None,
        scope_of_work: str = None,
        notice_period: float = None,
        contract_template: ContractTemplate = None,
        custom_fields: List[ContractCustomField] = None,
        quote: EorQuoteBase = None,
        external_id: str = None,
    ):
        self.id_ = id_
        self.title = title
        self.type_ = self._enum_matching(type_, ContractTypeEnum.list(), "type_")
        self.status = self._enum_matching(status, ContractStatusEnum.list(), "status")
        self.created_at = created_at
        if job_title is not None:
            self.job_title = job_title
        self.seniority = self._define_object(seniority, Seniority)
        if who_reports is not None:
            self.who_reports = self._enum_matching(
                who_reports, ContractWhoReportsEnum.list(), "who_reports"
            )
        if scope_of_work is not None:
            self.scope_of_work = scope_of_work
        self.start_date = start_date
        self.termination_date = termination_date
        self.special_clause = special_clause
        self.is_archived = is_archived
        self.client = self._define_object(client, ClientOfContract)
        self.worker = self._define_object(worker, WorkerOfContract)
        self.invitations = self._define_object(invitations, InvitationsOfBasicContract)
        self.signatures = self._define_object(signatures, SignaturesOfContract)
        self.compensation_details = self._define_object(
            compensation_details, CompensationDetailsOfContract
        )
        self.employment_details = self._define_object(
            employment_details, EmploymentDetailsOfContract
        )
        if notice_period is not None:
            self.notice_period = notice_period
        if contract_template is not None:
            self.contract_template = self._define_object(
                contract_template, ContractTemplate
            )
        if custom_fields is not None:
            self.custom_fields = self._define_list(custom_fields, ContractCustomField)
        if quote is not None:
            self.quote = self._define_object(quote, EorQuoteBase)
        if external_id is not None:
            self.external_id = external_id



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .timesheet_status_enum import TimesheetStatusEnum


@JsonMap({"id_": "id"})
class TaskCreated(BaseModel):
    """TaskCreated

    :param created: Whether the creation operation was successful., defaults to None
    :type created: bool, optional
    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param status: Status of this entry., defaults to None
    :type status: TimesheetStatusEnum, optional
    """

    def __init__(
        self, created: bool = None, id_: str = None, status: TimesheetStatusEnum = None
    ):
        if created is not None:
            self.created = created
        if id_ is not None:
            self.id_ = id_
        if status is not None:
            self.status = self._enum_matching(
                status, TimesheetStatusEnum.list(), "status"
            )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .marital_status_enum import MaritalStatusEnum


@JsonMap({})
class GpEmployeeInformationUpdateData(BaseModel):
    """GpEmployeeInformationUpdateData

    :param first_name: First name of the employee., defaults to None
    :type first_name: str, optional
    :param middle_name: Middle name of the employee., defaults to None
    :type middle_name: str, optional
    :param last_name: Last name of the employee., defaults to None
    :type last_name: str, optional
    :param date_of_birth: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type date_of_birth: str, optional
    :param gender: Employee's gender., defaults to None
    :type gender: str, optional
    :param marital_status: marital_status, defaults to None
    :type marital_status: MaritalStatusEnum, optional
    :param employee_number: Employee's number., defaults to None
    :type employee_number: str, optional
    """

    def __init__(
        self,
        first_name: str = None,
        middle_name: str = None,
        last_name: str = None,
        date_of_birth: str = None,
        gender: str = None,
        marital_status: MaritalStatusEnum = None,
        employee_number: str = None,
    ):
        if first_name is not None:
            self.first_name = first_name
        if middle_name is not None:
            self.middle_name = middle_name
        if last_name is not None:
            self.last_name = last_name
        if date_of_birth is not None:
            self.date_of_birth = self._pattern_matching(
                date_of_birth, "^\d{4}-\d{2}-\d{2}$", "date_of_birth"
            )
        if gender is not None:
            self.gender = gender
        if marital_status is not None:
            self.marital_status = self._enum_matching(
                marital_status, MaritalStatusEnum.list(), "marital_status"
            )
        if employee_number is not None:
            self.employee_number = employee_number



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"is_verified": "isVerified"})
class AlternateEmailItem(BaseModel):
    """AlternateEmailItem

    :param email: User's email address.
    :type email: str
    :param is_verified: is_verified
    :type is_verified: bool
    """

    def __init__(self, email: str, is_verified: bool):
        self.email = email
        self.is_verified = is_verified



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .team_of_basic_contract import TeamOfBasicContract


@JsonMap({})
class ClientOfBasicContract(BaseModel):
    """ClientOfBasicContract

    :param team: team, defaults to None
    :type team: TeamOfBasicContract, optional
    """

    def __init__(self, team: TeamOfBasicContract = None):
        if team is not None:
            self.team = self._define_object(team, TeamOfBasicContract)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class BankAccountValueAllowed(BaseModel):
    """BankAccountValueAllowed

    :param value: The key of the allowed value., defaults to None
    :type value: str, optional
    :param label: The name of the allowed value., defaults to None
    :type label: str, optional
    """

    def __init__(self, value: str = None, label: str = None):
        if value is not None:
            self.value = value
        if label is not None:
            self.label = label



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .timesheet_status_enum import TimesheetStatusEnum
from .file_attachment_info import FileAttachmentInfo
from .timesheet_approver import TimesheetApprover
from .contract_type_enum import ContractTypeEnum


@JsonMap({})
class TimesheetWorksheet(BaseModel):
    """TimesheetWorksheet

    :param weeks: weeks
    :type weeks: float
    :param days: days
    :type days: float
    :param hours: hours
    :type hours: float
    :param minutes: minutes
    :type minutes: float
    """

    def __init__(self, weeks: float, days: float, hours: float, minutes: float):
        self.weeks = weeks
        self.days = days
        self.hours = hours
        self.minutes = minutes


@JsonMap({"id_": "id"})
class TimesheetReviewedBy(BaseModel):
    """TimesheetReviewedBy

    :param id_: id_
    :type id_: IdentifierValue
    :param full_name: full_name
    :type full_name: str
    :param reviewed_at: reviewed_at
    :type reviewed_at: str
    :param remarks: remarks
    :type remarks: str
    """

    def __init__(
        self, id_: IdentifierValue, full_name: str, reviewed_at: str, remarks: str
    ):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.full_name = full_name
        self.reviewed_at = reviewed_at
        self.remarks = remarks


@JsonMap({"id_": "id", "type_": "type"})
class TimesheetContract(BaseModel):
    """TimesheetContract

    :param id_: id_
    :type id_: str
    :param title: title
    :type title: str
    :param type_: Type of a contract
    :type type_: ContractTypeEnum
    """

    def __init__(self, id_: str, title: str, type_: ContractTypeEnum):
        self.id_ = id_
        self.title = title
        self.type_ = self._enum_matching(type_, ContractTypeEnum.list(), "type_")


@JsonMap({})
class TimesheetPaymentCycle(BaseModel):
    """TimesheetPaymentCycle

    :param start_date: Long date-time format following ISO-8601, defaults to None
    :type start_date: str, optional
    :param end_date: Long date-time format following ISO-8601, defaults to None
    :type end_date: str, optional
    """

    def __init__(self, start_date: str = None, end_date: str = None):
        if start_date is not None:
            self.start_date = start_date
        if end_date is not None:
            self.end_date = end_date


@JsonMap({"id_": "id"})
class TimesheetReportedBy(BaseModel):
    """TimesheetReportedBy

    :param id_: id_
    :type id_: IdentifierValue
    :param full_name: full_name
    :type full_name: str
    """

    def __init__(self, id_: IdentifierValue, full_name: str):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.full_name = full_name


@JsonMap({"id_": "id", "type_": "type"})
class Timesheet(BaseModel):
    """Timesheet

    :param id_: id_
    :type id_: IdentifierValue
    :param type_: Deprecated - it is always "work"
    :type type_: str
    :param status: Status of this entry.
    :type status: TimesheetStatusEnum
    :param description: description
    :type description: str
    :param date_submitted: Long date-time format following ISO-8601
    :type date_submitted: str
    :param currency_code: currency_code
    :type currency_code: str
    :param total_amount: is equal to quantity times rate of active work statement + bonus in this record
    :type total_amount: str
    :param quantity: quantity
    :type quantity: float
    :param created_at: created_at
    :type created_at: str
    :param scale: scale, defaults to None
    :type scale: str, optional
    :param custom_scale: custom_scale, defaults to None
    :type custom_scale: str, optional
    :param attachment: This object is used for linking file attachments to your records.
    :type attachment: FileAttachmentInfo
    :param worksheet: worksheet
    :type worksheet: TimesheetWorksheet
    :param reviewed_by: reviewed_by
    :type reviewed_by: TimesheetReviewedBy
    :param contract: contract
    :type contract: TimesheetContract
    :param payment_cycle: payment_cycle, defaults to None
    :type payment_cycle: TimesheetPaymentCycle, optional
    :param reported_by: reported_by
    :type reported_by: TimesheetReportedBy
    :param approvers: approvers
    :type approvers: List[TimesheetApprover]
    """

    def __init__(
        self,
        id_: IdentifierValue,
        type_: str,
        status: TimesheetStatusEnum,
        description: str,
        date_submitted: str,
        currency_code: str,
        total_amount: str,
        quantity: float,
        created_at: str,
        attachment: FileAttachmentInfo,
        worksheet: TimesheetWorksheet,
        reviewed_by: TimesheetReviewedBy,
        contract: TimesheetContract,
        reported_by: TimesheetReportedBy,
        approvers: List[TimesheetApprover],
        scale: str = None,
        custom_scale: str = None,
        payment_cycle: TimesheetPaymentCycle = None,
    ):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.type_ = type_
        self.status = self._enum_matching(status, TimesheetStatusEnum.list(), "status")
        self.description = description
        self.date_submitted = date_submitted
        self.currency_code = self._pattern_matching(
            currency_code, "^[A-Z]{3}$", "currency_code"
        )
        self.total_amount = total_amount
        self.quantity = quantity
        self.created_at = created_at
        if scale is not None:
            self.scale = scale
        if custom_scale is not None:
            self.custom_scale = custom_scale
        self.attachment = self._define_object(attachment, FileAttachmentInfo)
        self.worksheet = self._define_object(worksheet, TimesheetWorksheet)
        self.reviewed_by = self._define_object(reviewed_by, TimesheetReviewedBy)
        self.contract = self._define_object(contract, TimesheetContract)
        if payment_cycle is not None:
            self.payment_cycle = self._define_object(
                payment_cycle, TimesheetPaymentCycle
            )
        self.reported_by = self._define_object(reported_by, TimesheetReportedBy)
        self.approvers = self._define_list(approvers, TimesheetApprover)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class GpContractStatusTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar FULL_TIME: "Full-time"
    :vartype FULL_TIME: str
    :cvar PART_TIME: "Part-time"
    :vartype PART_TIME: str
    """

    FULL_TIME = "Full-time"
    PART_TIME = "Part-time"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, GpContractStatusTypeEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .requester_time_off import RequesterTimeOff


class EmployeeTimeoffsItemResponseType(Enum):
    """An enumeration representing different categories.

    :cvar VACATION: "VACATION"
    :vartype VACATION: str
    :cvar SICK_LEAVE: "SICK_LEAVE"
    :vartype SICK_LEAVE: str
    :cvar OTHER: "OTHER"
    :vartype OTHER: str
    """

    VACATION = "VACATION"
    SICK_LEAVE = "SICK_LEAVE"
    OTHER = "OTHER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                EmployeeTimeoffsItemResponseType._member_map_.values(),
            )
        )


@JsonMap({"id_": "id"})
class EmployeeTimeoffsItemResponseAttachments(BaseModel):
    """EmployeeTimeoffsItemResponseAttachments

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param time_off_id: time_off_id, defaults to None
    :type time_off_id: str, optional
    :param filename: filename, defaults to None
    :type filename: str, optional
    :param created_at: created_at, defaults to None
    :type created_at: str, optional
    :param updated_at: updated_at, defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        time_off_id: str = None,
        filename: str = None,
        created_at: str = None,
        updated_at: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if time_off_id is not None:
            self.time_off_id = time_off_id
        if filename is not None:
            self.filename = filename
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at


@JsonMap({"type_": "type"})
class EmployeeTimeoffsItemResponse(BaseModel):
    """EmployeeTimeoffsItemResponse

    :param time_off_id: time_off_id, defaults to None
    :type time_off_id: str, optional
    :param time_off_type_id: time_off_type_id, defaults to None
    :type time_off_type_id: str, optional
    :param policy_id: policy_id, defaults to None
    :type policy_id: str, optional
    :param policy_name: policy_name, defaults to None
    :type policy_name: str, optional
    :param policy_type_name: policy_type_name, defaults to None
    :type policy_type_name: str, optional
    :param reason: reason, defaults to None
    :type reason: str, optional
    :param type_: Time off type., defaults to None
    :type type_: EmployeeTimeoffsItemResponseType, optional
    :param requested_at: requested_at, defaults to None
    :type requested_at: str, optional
    :param reviewed_at: reviewed_at, defaults to None
    :type reviewed_at: str, optional
    :param with_multiple_dates: with_multiple_dates, defaults to None
    :type with_multiple_dates: bool, optional
    :param single_date: single_date, defaults to None
    :type single_date: str, optional
    :param created_at: created_at, defaults to None
    :type created_at: str, optional
    :param updated_at: updated_at, defaults to None
    :type updated_at: str, optional
    :param denial_reason: denial_reason, defaults to None
    :type denial_reason: bool, optional
    :param requester: requester, defaults to None
    :type requester: RequesterTimeOff, optional
    :param days_used_start_year: days_used_start_year, defaults to None
    :type days_used_start_year: str, optional
    :param days_used_end_year: days_used_end_year, defaults to None
    :type days_used_end_year: str, optional
    :param days_used: days_used, defaults to None
    :type days_used: str, optional
    :param start_date: start_date, defaults to None
    :type start_date: str, optional
    :param end_date: end_date, defaults to None
    :type end_date: str, optional
    :param is_start_date_half_day: is_start_date_half_day, defaults to None
    :type is_start_date_half_day: bool, optional
    :param is_end_date_half_day: is_end_date_half_day, defaults to None
    :type is_end_date_half_day: bool, optional
    :param attachments: attachments, defaults to None
    :type attachments: List[EmployeeTimeoffsItemResponseAttachments], optional
    :param change_request: change_request, defaults to None
    :type change_request: str, optional
    :param is_updated: is_updated, defaults to None
    :type is_updated: bool, optional
    :param applied_changes: applied_changes, defaults to None
    :type applied_changes: List[any], optional
    :param can_request_changes: can_request_changes, defaults to None
    :type can_request_changes: bool, optional
    """

    def __init__(
        self,
        time_off_id: str = None,
        time_off_type_id: str = None,
        policy_id: str = None,
        policy_name: str = None,
        policy_type_name: str = None,
        reason: str = None,
        type_: EmployeeTimeoffsItemResponseType = None,
        requested_at: str = None,
        reviewed_at: str = None,
        with_multiple_dates: bool = None,
        single_date: str = None,
        created_at: str = None,
        updated_at: str = None,
        denial_reason: bool = None,
        requester: RequesterTimeOff = None,
        days_used_start_year: str = None,
        days_used_end_year: str = None,
        days_used: str = None,
        start_date: str = None,
        end_date: str = None,
        is_start_date_half_day: bool = None,
        is_end_date_half_day: bool = None,
        attachments: List[EmployeeTimeoffsItemResponseAttachments] = None,
        change_request: str = None,
        is_updated: bool = None,
        applied_changes: List[any] = None,
        can_request_changes: bool = None,
    ):
        if time_off_id is not None:
            self.time_off_id = time_off_id
        if time_off_type_id is not None:
            self.time_off_type_id = time_off_type_id
        if policy_id is not None:
            self.policy_id = policy_id
        if policy_name is not None:
            self.policy_name = policy_name
        if policy_type_name is not None:
            self.policy_type_name = policy_type_name
        if reason is not None:
            self.reason = reason
        if type_ is not None:
            self.type_ = self._enum_matching(
                type_, EmployeeTimeoffsItemResponseType.list(), "type_"
            )
        if requested_at is not None:
            self.requested_at = requested_at
        if reviewed_at is not None:
            self.reviewed_at = reviewed_at
        if with_multiple_dates is not None:
            self.with_multiple_dates = with_multiple_dates
        if single_date is not None:
            self.single_date = single_date
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if denial_reason is not None:
            self.denial_reason = denial_reason
        if requester is not None:
            self.requester = self._define_object(requester, RequesterTimeOff)
        if days_used_start_year is not None:
            self.days_used_start_year = days_used_start_year
        if days_used_end_year is not None:
            self.days_used_end_year = days_used_end_year
        if days_used is not None:
            self.days_used = days_used
        if start_date is not None:
            self.start_date = start_date
        if end_date is not None:
            self.end_date = end_date
        if is_start_date_half_day is not None:
            self.is_start_date_half_day = is_start_date_half_day
        if is_end_date_half_day is not None:
            self.is_end_date_half_day = is_end_date_half_day
        if attachments is not None:
            self.attachments = self._define_list(
                attachments, EmployeeTimeoffsItemResponseAttachments
            )
        if change_request is not None:
            self.change_request = change_request
        if is_updated is not None:
            self.is_updated = is_updated
        if applied_changes is not None:
            self.applied_changes = applied_changes
        if can_request_changes is not None:
            self.can_request_changes = can_request_changes



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ProfileTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar CLIENT: "client"
    :vartype CLIENT: str
    :cvar CONTRACTOR: "contractor"
    :vartype CONTRACTOR: str
    :cvar ICP: "icp"
    :vartype ICP: str
    :cvar DIRECT_EMPLOYEE: "direct_employee"
    :vartype DIRECT_EMPLOYEE: str
    :cvar HRIS_DIRECT_EMPLOYEE: "hris_direct_employee"
    :vartype HRIS_DIRECT_EMPLOYEE: str
    :cvar SERVICE_ACCOUNT: "service_account"
    :vartype SERVICE_ACCOUNT: str
    """

    CLIENT = "client"
    CONTRACTOR = "contractor"
    ICP = "icp"
    DIRECT_EMPLOYEE = "direct_employee"
    HRIS_DIRECT_EMPLOYEE = "hris_direct_employee"
    SERVICE_ACCOUNT = "service_account"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ProfileTypeEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class Documents(BaseModel):
    """Documents

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param name: The name of the document., defaults to None
    :type name: str, optional
    :param description: A short description of the document., defaults to None
    :type description: str, optional
    :param has_template: Denotes if this document has a template available for download, defaults to None
    :type has_template: bool, optional
    :param fillable: Denotes if the document is available to fill out on Deel., defaults to None
    :type fillable: bool, optional
    :param is_optional: Denotes if the document is optional or required., defaults to None
    :type is_optional: bool, optional
    :param country: The country associated with this document., defaults to None
    :type country: str, optional
    :param uploaded_at: The timestamp when this document was uploaded / supplied to Deel., defaults to None
    :type uploaded_at: str, optional
    :param filenames: The list of filenames for the associated uploaded documents., defaults to None
    :type filenames: List[str], optional
    """

    def __init__(
        self,
        id_: IdentifierValue = None,
        name: str = None,
        description: str = None,
        has_template: bool = None,
        fillable: bool = None,
        is_optional: bool = None,
        country: str = None,
        uploaded_at: str = None,
        filenames: List[str] = None,
    ):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if has_template is not None:
            self.has_template = has_template
        if fillable is not None:
            self.fillable = fillable
        if is_optional is not None:
            self.is_optional = is_optional
        if country is not None:
            self.country = country
        if uploaded_at is not None:
            self.uploaded_at = uploaded_at
        if filenames is not None:
            self.filenames = filenames


@JsonMap({})
class GetEmployeeComplianceDocumentsContainerData(BaseModel):
    """GetEmployeeComplianceDocumentsContainerData

    :param documents: documents
    :type documents: List[Documents]
    """

    def __init__(self, documents: List[Documents]):
        self.documents = self._define_list(documents, Documents)


@JsonMap({})
class GetEmployeeComplianceDocumentsContainer(BaseModel):
    """GetEmployeeComplianceDocumentsContainer

    :param data: data
    :type data: GetEmployeeComplianceDocumentsContainerData
    """

    def __init__(self, data: GetEmployeeComplianceDocumentsContainerData):
        self.data = self._define_object(
            data, GetEmployeeComplianceDocumentsContainerData
        )



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class BenefitStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar REQUIRED: "REQUIRED"
    :vartype REQUIRED: str
    :cvar ENABLED: "ENABLED"
    :vartype ENABLED: str
    :cvar DISABLED: "DISABLED"
    :vartype DISABLED: str
    """

    REQUIRED = "REQUIRED"
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, BenefitStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .invoice_list_container import InvoiceListContainer
from .get_invoice_list_entities import GetInvoiceListEntities
from .deel_invoice_list_container import DeelInvoiceListContainer
from .invoice_download_link_container import InvoiceDownloadLinkContainer
from .payment_list_container import PaymentListContainer
from .get_payment_list_currencies import GetPaymentListCurrencies
from .get_payment_list_entities import GetPaymentListEntities
from .payment_break_down_container import PaymentBreakDownContainer
from .admin_users_container import AdminUsersContainer
from .admin_user_create_container import AdminUserCreateContainer
from .create_admin_user_response_container import CreateAdminUserResponseContainer
from .hris_direct_employee import HrisDirectEmployee
from .hris_direct_employee_container import HrisDirectEmployeeContainer
from .internal_people_container import InternalPeopleContainer
from .people_container import PeopleContainer
from .people_sort_by_enum import PeopleSortByEnum
from .sort_dir_enum import SortDirEnum
from .hiring_status_enum import HiringStatusEnum
from .people_by_id_container import PeopleByIdContainer
from .update_worker_department_container import UpdateWorkerDepartmentContainer
from .generic_result_updated import GenericResultUpdated
from .update_worker_working_location_container import (
    UpdateWorkerWorkingLocationContainer,
)
from .people_me import PeopleMe
from .employee_timeoffs_container import EmployeeTimeoffsContainer
from .create_people_timeoff import CreatePeopleTimeoff
from .employee_timeoffs_creation_container import EmployeeTimeoffsCreationContainer
from .employee_timeoffs_entitlements_container import (
    EmployeeTimeoffsEntitlementsContainer,
)
from .people_time_off_container import PeopleTimeOffContainer
from .generic_result_deleted import GenericResultDeleted
from .timeoff_to_review_internal_container import TimeoffToReviewInternalContainer
from .employee_timeoffs_policies_container import EmployeeTimeoffsPoliciesContainer
from .eor_country_validations_container import EorCountryValidationsContainer
from .eor_payslips_list_container import EorPayslipsListContainer
from .eor_payslip_download_url_container import EorPayslipDownloadUrlContainer
from .eor_employee_cost_calculation_request_body_container import (
    EorEmployeeCostCalculationRequestBodyContainer,
)
from .eor_employee_cost_calculation_response_container import (
    EorEmployeeCostCalculationResponseContainer,
)
from .eor_contract_benefits_container import EorContractBenefitsContainer
from .eor_contract_to_create_container import EorContractToCreateContainer
from .eor_contract_created_container import EorContractCreatedContainer
from .gp_contract_to_create_container import GpContractToCreateContainer
from .gp_contract_created_container import GpContractCreatedContainer
from .gp_payslips_list_container import GpPayslipsListContainer
from .gp_employee_address_to_update_container import GpEmployeeAddressToUpdateContainer
from .gp_employee_address_updated_container import GpEmployeeAddressUpdatedContainer
from .worker_bank_accounts_info_container import WorkerBankAccountsInfoContainer
from .add_worker_bank_account_container import AddWorkerBankAccountContainer
from .bank_account_updated_container import BankAccountUpdatedContainer
from .bank_account_guide_container import BankAccountGuideContainer
from .gp_employee_compensation_to_update_container import (
    GpEmployeeCompensationToUpdateContainer,
)
from .gp_employee_compensation_updated_container import (
    GpEmployeeCompensationUpdatedContainer,
)
from .gp_employee_pto_to_update_container import GpEmployeePtoToUpdateContainer
from .gp_employee_information_to_update_container import (
    GpEmployeeInformationToUpdateContainer,
)
from .gp_employee_information_updated_container import (
    GpEmployeeInformationUpdatedContainer,
)
from .gp_payslip_download_url_container import GpPayslipDownloadUrlContainer
from .gp_payroll_event_report_container import GpPayrollEventReportContainer
from .global_payroll_g2_n_report_container import GlobalPayrollG2NReportContainer
from .worker_termination_body_container import WorkerTerminationBodyContainer
from .worker_termination_container import WorkerTerminationContainer
from .contract_to_create_container_pay_as_you_go_time_based import (
    ContractToCreateContainerPayAsYouGoTimeBased,
)
from .contract_container_pay_as_you_go_time_based import (
    ContractContainerPayAsYouGoTimeBased,
)
from .contract_to_create_container_payg_tasks import ContractToCreateContainerPaygTasks
from .contract_container_payg_tasks import ContractContainerPaygTasks
from .contract_to_create_container_payg_milestones import (
    ContractToCreateContainerPaygMilestones,
)
from .contract_container_payg_milestones import ContractContainerPaygMilestones
from .contract_to_create_container_ongoing_time_based import (
    ContractToCreateContainerOngoingTimeBased,
)
from .contract_container_ongoing_time_based import ContractContainerOngoingTimeBased
from .contract_to_create_container import ContractToCreateContainer
from .contract_container import ContractContainer
from .contract_to_amend_details_container import ContractToAmendDetailsContainer
from .generic_result_created import GenericResultCreated
from .contract_to_terminate_container import ContractToTerminateContainer
from .contract_termination_result_container import ContractTerminationResultContainer
from .premium_to_add_container import PremiumToAddContainer
from .premium_result_added_container import PremiumResultAddedContainer
from .adjustment_to_create_container import AdjustmentToCreateContainer
from .adjustment_created_container import AdjustmentCreatedContainer
from .adjustment_to_update_container import AdjustmentToUpdateContainer
from .adjustments_categories_container import AdjustmentsCategoriesContainer
from .input_to_create_file_ref import InputToCreateFileRef
from .output_to_create_file_ref_container import OutputToCreateFileRefContainer
from .adjustments_container import AdjustmentsContainer
from .candidate_to_create_container import CandidateToCreateContainer
from .candidate_to_patch_container import CandidateToPatchContainer
from .additional_eor_info_container import AdditionalEorInfoContainer
from .employee_contract_signature_to_create_container import (
    EmployeeContractSignatureToCreateContainer,
)
from .request_custom_verification_letter_container import (
    RequestCustomVerificationLetterContainer,
)
from .request_custom_verification_letter_with_file import (
    RequestCustomVerificationLetterWithFile,
)
from .hr_verification_letters_and_documents_list_container import (
    HrVerificationLettersAndDocumentsListContainer,
)
from .employee_agreement_download_container import EmployeeAgreementDownloadContainer
from .bank_account_to_add_container import BankAccountToAddContainer
from .bank_account_added_container import BankAccountAddedContainer
from .employee_payslips_list_container import EmployeePayslipsListContainer
from .get_employee_compliance_documents_container import (
    GetEmployeeComplianceDocumentsContainer,
)
from .upload_employee_compliance_document_file_container import (
    UploadEmployeeComplianceDocumentFileContainer,
)
from .upload_employee_compliance_document_container import (
    UploadEmployeeComplianceDocumentContainer,
)
from .get_employee_compliance_document_template_download_link_container import (
    GetEmployeeComplianceDocumentTemplateDownloadLinkContainer,
)
from .employee_tax_documents_list_container import EmployeeTaxDocumentsListContainer
from .contract_list_container import ContractListContainer
from .contract_type_enum import ContractTypeEnum
from .contract_status_enum import ContractStatusEnum
from .get_contract_list_currencies import GetContractListCurrencies
from .contracts_sort_by_enum import ContractsSortByEnum
from .input_to_patch_contract_external_id import InputToPatchContractExternalId
from .file_object import FileObject
from .contract_document_container import ContractDocumentContainer
from .alternate_email_item import AlternateEmailItem
from .contract_signature_to_create_container import ContractSignatureToCreateContainer
from .contract_invitation_to_create_container import ContractInvitationToCreateContainer
from .final_payment_calculated_container import FinalPaymentCalculatedContainer
from .calculate_final_payment_calculation_type import (
    CalculateFinalPaymentCalculationType,
)
from .estimate_first_payment_container import EstimateFirstPaymentContainer
from .response_estimate_first_payment_container import (
    ResponseEstimateFirstPaymentContainer,
)
from .contract_template_list_container import ContractTemplateListContainer
from .worker_documents_by_id_container import WorkerDocumentsByIdContainer
from .download_worker_documents_by_id_container import (
    DownloadWorkerDocumentsByIdContainer,
)
from .task_list_container import TaskListContainer
from .input_to_create_pgo_task import InputToCreatePgoTask
from .task_created_container import TaskCreatedContainer
from .request_body_to_create_pgo_task_reviews_reviews_container import (
    RequestBodyToCreatePgoTaskReviewsReviewsContainer,
)
from .request_body_to_create_pgo_task_reviews_by_id_reviews_container import (
    RequestBodyToCreatePgoTaskReviewsByIdReviewsContainer,
)
from .timesheet_list_container import TimesheetListContainer
from .timesheet_status_enum import TimesheetStatusEnum
from .timesheet_to_create_container import TimesheetToCreateContainer
from .timesheet_to_create_container_with_file import TimesheetToCreateContainerWithFile
from .invoice_adjustment_created_container import InvoiceAdjustmentCreatedContainer
from .timesheet_container import TimesheetContainer
from .timesheet_to_update_container import TimesheetToUpdateContainer
from .timesheet_review_to_create_container import TimesheetReviewToCreateContainer
from .timesheet_reviews_to_create_container import TimesheetReviewsToCreateContainer
from .milestone_list_container import MilestoneListContainer
from .milestone_to_create_container import MilestoneToCreateContainer
from .milestone_to_create_form_with_file import MilestoneToCreateFormWithFile
from .milestone_container import MilestoneContainer
from .milestone_review_to_create_container import MilestoneReviewToCreateContainer
from .milestone_reviews_to_create_container import MilestoneReviewsToCreateContainer
from .off_cycle_payment_list_container import OffCyclePaymentListContainer
from .off_cycle_payment_to_create_container import OffCyclePaymentToCreateContainer
from .generic_result_created_with_id import GenericResultCreatedWithId
from .off_cycle_payment_container import OffCyclePaymentContainer
from .eor_timeoffs_container import EorTimeoffsContainer
from .create_timeoff_container import CreateTimeoffContainer
from .eor_timeoffs_item_container import EorTimeoffsItemContainer
from .eor_entitlements_container import EorEntitlementsContainer
from .eor_client_timeoffs_container import EorClientTimeoffsContainer
from .timeoff_to_review_container import TimeoffToReviewContainer
from .invoice_adjustment_list_container import InvoiceAdjustmentListContainer
from .invoice_adjustment_type_enum import InvoiceAdjustmentTypeEnum
from .invoice_adjustment_status_enum import InvoiceAdjustmentStatusEnum
from .invoice_adjustment_to_create_container import InvoiceAdjustmentToCreateContainer
from .invoice_adjustment_to_create_with_file import InvoiceAdjustmentToCreateWithFile
from .invoice_adjustment_to_update_container import InvoiceAdjustmentToUpdateContainer
from .invoice_adjustment_review_to_create_container import (
    InvoiceAdjustmentReviewToCreateContainer,
)
from .invoice_adjustment_reviews_to_create_container import (
    InvoiceAdjustmentReviewsToCreateContainer,
)
from .invoice_adjustment_attachment_container import (
    InvoiceAdjustmentAttachmentContainer,
)
from .legal_entity_list_container import LegalEntityListContainer
from .organization_list_container import OrganizationListContainer
from .team_list_container import TeamListContainer
from .agreement_list_container import AgreementListContainer
from .departments_container import DepartmentsContainer
from .working_locations_container import WorkingLocationsContainer
from .country_list_container import CountryListContainer
from .currency_list_container import CurrencyListContainer
from .job_title_list_container import JobTitleListContainer
from .seniority_list_container import SeniorityListContainer
from .timeoff_type_list_container import TimeoffTypeListContainer
from .webhook_list_response import WebhookListResponse
from .create_webhook_request import CreateWebhookRequest
from .webhook_item_response import WebhookItemResponse
from .patch_webhook_request import PatchWebhookRequest
from .webhook_event_type_list_response import WebhookEventTypeListResponse
from .create_public_token_container import CreatePublicTokenContainer
from .public_token_created_container import PublicTokenCreatedContainer
from .equity_stakeholders_container import EquityStakeholdersContainer
from .invoice import Invoice
from .page_info_without_cursor_new import PageInfoWithoutCursorNew
from .invoice_status_enum import InvoiceStatusEnum
from .legal_entity_type import LegalEntityType
from .deel_invoice import DeelInvoice
from .invoice_download_object import InvoiceDownloadObject
from .payment_object import PaymentObject
from .payment import Payment
from .payment_method import PaymentMethod
from .payment_status_enum import PaymentStatusEnum
from .payment_worker import PaymentWorker
from .payment_method_enum import PaymentMethodEnum
from .payment_break_down import PaymentBreakDown
from .admin_user import AdminUser
from .admin_user_create_request import AdminUserCreateRequest
from .hris_direct_employee_details import HrisDirectEmployeeDetails
from .hris_team_information import HrisTeamInformation
from .hris_compensation import HrisCompensation
from .hris_job_information_title_id import HrisJobInformationTitleId
from .hris_job_information_title_name import HrisJobInformationTitleName
from .hris_contract_part_time import HrisContractPartTime
from .hris_contract_full_time import HrisContractFullTime
from .hris_direct_employee_response import HrisDirectEmployeeResponse
from .internal_people import InternalPeople
from .people_client_legal_entity import PeopleClientLegalEntity
from .monthly_payment import MonthlyPayment
from .employment import Employment
from .identifier_value import IdentifierValue
from .team import Team
from .people_payment import PeoplePayment
from .employee import Employee
from .address import Address
from .email import Email
from .employment_detail import EmploymentDetail
from .people_custom_field import PeopleCustomField
from .update_worker_department import UpdateWorkerDepartment
from .update_worker_working_location import UpdateWorkerWorkingLocation
from .profile_type_enum import ProfileTypeEnum
from .employee_timeoffs_item import EmployeeTimeoffsItem
from .employee_timeoffs_item_response import EmployeeTimeoffsItemResponse
from .requester_time_off import RequesterTimeOff
from .employee_timeoffs_entitlements_item import EmployeeTimeoffsEntitlementsItem
from .timeoff_review_internal import TimeoffReviewInternal
from .employee_timeoffs_policies_item import EmployeeTimeoffsPoliciesItem
from .eor_country_validations import EorCountryValidations
from .eor_field import EorField
from .salary_frequency_scale_enum import SalaryFrequencyScaleEnum
from .benefit_status_enum import BenefitStatusEnum
from .health_insurance_provider import HealthInsuranceProvider
from .pension_provider import PensionProvider
from .contribution import Contribution
from .benefit_contribution_type_enum import BenefitContributionTypeEnum
from .payslips import Payslips
from .payslip_download_url import PayslipDownloadUrl
from .eor_employee_cost_calculation_request_body import (
    EorEmployeeCostCalculationRequestBody,
)
from .eor_contract_benefits import EorContractBenefits
from .eor_contract_to_create import EorContractToCreate
from .pension_eor_contract_to_create import PensionEorContractToCreate
from .eor_contract_created import EorContractCreated
from .seniority import Seniority
from .gp_contract_to_create import GpContractToCreate
from .gp_client import GpClient
from .gp_contract_status_type_enum import GpContractStatusTypeEnum
from .gp_contract_created import GpContractCreated
from .gp_employee_address_update_data import GpEmployeeAddressUpdateData
from .gp_employee_address_updated import GpEmployeeAddressUpdated
from .worker_bank_account_info import WorkerBankAccountInfo
from .worker_bank_account_to_add import WorkerBankAccountToAdd
from .bank_account_updated import BankAccountUpdated
from .bank_account_guide import BankAccountGuide
from .bank_account_value_allowed import BankAccountValueAllowed
from .validation_type import ValidationType
from .validation_type_enum import ValidationTypeEnum
from .gp_employee_compensation_update_data import GpEmployeeCompensationUpdateData
from .gp_contract_salary_scale_enum import GpContractSalaryScaleEnum
from .gp_employee_compensation_updated import GpEmployeeCompensationUpdated
from .gp_contract_salary_status_enum import GpContractSalaryStatusEnum
from .gp_employee_pto_update_data import GpEmployeePtoUpdateData
from .gp_employee_information_update_data import GpEmployeeInformationUpdateData
from .marital_status_enum import MaritalStatusEnum
from .gp_payroll_event_reports import GpPayrollEventReports
from .global_payroll_g2_n_report import GlobalPayrollG2NReport
from .worker_termination_body import WorkerTerminationBody
from .worker_termination import WorkerTermination
from .contract_to_create_pay_as_you_go_time_based import (
    ContractToCreatePayAsYouGoTimeBased,
)
from .contract_who_reports_enum import ContractWhoReportsEnum
from .meta_data_of_contract_to_create import MetaDataOfContractToCreate
from .work_statement_cycle_scale_enum import WorkStatementCycleScaleEnum
from .work_statement_cycle_end_type_enum import WorkStatementCycleEndTypeEnum
from .work_statement_payment_due_type_enum import WorkStatementPaymentDueTypeEnum
from .work_statement_scale_enum import WorkStatementScaleEnum
from .contract import Contract
from .client_of_contract import ClientOfContract
from .worker_of_contract import WorkerOfContract
from .invitations_of_basic_contract import InvitationsOfBasicContract
from .signatures_of_contract import SignaturesOfContract
from .compensation_details_of_contract import CompensationDetailsOfContract
from .employment_details_of_contract import EmploymentDetailsOfContract
from .contract_template import ContractTemplate
from .contract_custom_field import ContractCustomField
from .eor_quote_base import EorQuoteBase
from .team_of_contract import TeamOfContract
from .client_of_contract_legal_entity_5 import ClientOfContractLegalEntity5
from .worker_legal_entity import WorkerLegalEntity
from .breakdown_costs_quote import BreakdownCostsQuote
from .cost_quote import CostQuote
from .contract_to_create_payg_tasks import ContractToCreatePaygTasks
from .compensation_details_of_contract_to_create_shared import (
    CompensationDetailsOfContractToCreateShared,
)
from .contract_to_create_payg_milestones import ContractToCreatePaygMilestones
from .contract_to_create_ongoing_time_based import ContractToCreateOngoingTimeBased
from .contract_to_create import ContractToCreate
from .contract_details_to_amend import ContractDetailsToAmend
from .contract_to_terminate import ContractToTerminate
from .contract_termination_result import ContractTerminationResult
from .premium_to_add import PremiumToAdd
from .premium_result_added import PremiumResultAdded
from .adjustment_to_create import AdjustmentToCreate
from .payroll_adjustment import PayrollAdjustment
from .adjustment_status_enum import AdjustmentStatusEnum
from .adjustment_to_update import AdjustmentToUpdate
from .adjustment_category import AdjustmentCategory
from .file_ref_type_enum import FileRefTypeEnum
from .output_to_create_file_ref import OutputToCreateFileRef
from .candidate_to_create import CandidateToCreate
from .candidate_status_enum import CandidateStatusEnum
from .candidate_to_patch import CandidateToPatch
from .employee_contract_signature_to_create import EmployeeContractSignatureToCreate
from .letter_request import LetterRequest
from .hr_document_info import HrDocumentInfo
from .employee_agreement_download_object import EmployeeAgreementDownloadObject
from .bank_account_to_add import BankAccountToAdd
from .bank_account_added import BankAccountAdded
from .bank_account_status import BankAccountStatus
from .employee_payslips_list import EmployeePayslipsList
from .employee_tax_documents_list import EmployeeTaxDocumentsList
from .basic_contract import BasicContract
from .page_info import PageInfo
from .client_of_basic_contract import ClientOfBasicContract
from .worker_of_basic_contract import WorkerOfBasicContract
from .signatures_of_basic_contract import SignaturesOfBasicContract
from .team_of_basic_contract import TeamOfBasicContract
from .contract_external_id_to_patch import ContractExternalIdToPatch
from .file_attachment_info import FileAttachmentInfo
from .contract_signature_to_create import ContractSignatureToCreate
from .contract_invitation_to_create import ContractInvitationToCreate
from .final_payment_calculated import FinalPaymentCalculated
from .estimate_first_payment import EstimateFirstPayment
from .contract_type_enum_for_estimate import ContractTypeEnumForEstimate
from .week_days_enum import WeekDaysEnum
from .response_estimate_first_payment import ResponseEstimateFirstPayment
from .pro_rata import ProRata
from .first_payment_date import FirstPaymentDate
from .contract_template_summary import ContractTemplateSummary
from .worker_document import WorkerDocument
from .worker_document_download_link import WorkerDocumentDownloadLink
from .pgo_task import PgoTask
from .pgo_task_to_create import PgoTaskToCreate
from .task_created import TaskCreated
from .pgo_task_reviews_to_create import PgoTaskReviewsToCreate
from .pgo_task_reviews_by_id_to_create import PgoTaskReviewsByIdToCreate
from .basic_timesheet import BasicTimesheet
from .page_info_without_cursor import PageInfoWithoutCursor
from .timesheet_shared_properties import TimesheetSharedProperties
from .invoice_adjustment_created import InvoiceAdjustmentCreated
from .timesheet import Timesheet
from .timesheet_approver import TimesheetApprover
from .timesheet_to_update import TimesheetToUpdate
from .timesheet_review_to_create import TimesheetReviewToCreate
from .timesheet_reviews_to_create import TimesheetReviewsToCreate
from .timesheet_id_items import TimesheetIdItems
from .milestone import Milestone
from .milestone_properties import MilestoneProperties
from .milestone_review_to_create import MilestoneReviewToCreate
from .milestone_reviews_to_create import MilestoneReviewsToCreate
from .off_cycle_payment import OffCyclePayment
from .off_cycle_payment_to_create import OffCyclePaymentToCreate
from .eor_timeoffs_base_item import EorTimeoffsBaseItem
from .timeoffs_type_enum import TimeoffsTypeEnum
from .timeoffs_status_enum import TimeoffsStatusEnum
from .timeoffs_profile import TimeoffsProfile
from .timeoffs_attachments_item import TimeoffsAttachmentsItem
from .create_timeoff import CreateTimeoff
from .eor_entitlements import EorEntitlements
from .eor_entitlement_list_item import EorEntitlementListItem
from .eor_holidays_rollover_type import EorHolidaysRolloverType
from .eor_client_timeoffs import EorClientTimeoffs
from .eor_timeoffs_employee_item import EorTimeoffsEmployeeItem
from .eor_client_timeoff_requests import EorClientTimeoffRequests
from .timeoff_review import TimeoffReview
from .basic_invoice_adjustment import BasicInvoiceAdjustment
from .invoice_adjustment_to_create import InvoiceAdjustmentToCreate
from .invoice_adjustment_type_to_create_enum import InvoiceAdjustmentTypeToCreateEnum
from .invoice_adjustment_to_update import InvoiceAdjustmentToUpdate
from .invoice_adjustment_review_to_create import InvoiceAdjustmentReviewToCreate
from .invoice_adjustment_reviews_to_create import InvoiceAdjustmentReviewsToCreate
from .attachment_file_ref import AttachmentFileRef
from .basic_legal_entity import BasicLegalEntity
from .basic_organization import BasicOrganization
from .basic_team import BasicTeam
from .agreement import Agreement
from .departments import Departments
from .working_locations import WorkingLocations
from .country import Country
from .state_of_country import StateOfCountry
from .currency import Currency
from .job_title import JobTitle
from .seniority_required import SeniorityRequired
from .webhook_item import WebhookItem
from .webhook_event_type_response import WebhookEventTypeResponse
from .create_public_token import CreatePublicToken
from .public_token import PublicToken
from .equity_stakeholder import EquityStakeholder



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.premium_to_add_container import PremiumToAddContainer
from ..models.premium_result_added_container import PremiumResultAddedContainer
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.generic_result_created import GenericResultCreated
from ..models.contract_to_terminate_container import ContractToTerminateContainer
from ..models.contract_to_create_container_payg_tasks import (
    ContractToCreateContainerPaygTasks,
)
from ..models.contract_to_create_container_payg_milestones import (
    ContractToCreateContainerPaygMilestones,
)
from ..models.contract_to_create_container_pay_as_you_go_time_based import (
    ContractToCreateContainerPayAsYouGoTimeBased,
)
from ..models.contract_to_create_container_ongoing_time_based import (
    ContractToCreateContainerOngoingTimeBased,
)
from ..models.contract_to_create_container import ContractToCreateContainer
from ..models.contract_to_amend_details_container import ContractToAmendDetailsContainer
from ..models.contract_termination_result_container import (
    ContractTerminationResultContainer,
)
from ..models.contract_container_payg_tasks import ContractContainerPaygTasks
from ..models.contract_container_payg_milestones import ContractContainerPaygMilestones
from ..models.contract_container_pay_as_you_go_time_based import (
    ContractContainerPayAsYouGoTimeBased,
)
from ..models.contract_container_ongoing_time_based import (
    ContractContainerOngoingTimeBased,
)
from ..models.contract_container import ContractContainer


class ContractorsService(BaseService):

    @cast_models
    def create_contract_time_based(
        self, request_body: ContractToCreateContainerPayAsYouGoTimeBased
    ) -> ContractContainerPayAsYouGoTimeBased:
        """Create a new contract (time-based).

        :param request_body: The request body.
        :type request_body: ContractToCreateContainerPayAsYouGoTimeBased
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractContainerPayAsYouGoTimeBased
        """

        Validator(ContractToCreateContainerPayAsYouGoTimeBased).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/time-based", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ContractContainerPayAsYouGoTimeBased._unmap(response)

    @cast_models
    def create_contract_task_based(
        self, request_body: ContractToCreateContainerPaygTasks
    ) -> ContractContainerPaygTasks:
        """Create a new Deel contract.

        :param request_body: The request body.
        :type request_body: ContractToCreateContainerPaygTasks
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractContainerPaygTasks
        """

        Validator(ContractToCreateContainerPaygTasks).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/task-based", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ContractContainerPaygTasks._unmap(response)

    @cast_models
    def create_contract_milestone_based(
        self, request_body: ContractToCreateContainerPaygMilestones
    ) -> ContractContainerPaygMilestones:
        """Create a new Deel contract.

        :param request_body: The request body.
        :type request_body: ContractToCreateContainerPaygMilestones
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractContainerPaygMilestones
        """

        Validator(ContractToCreateContainerPaygMilestones).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/milestone-based", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ContractContainerPaygMilestones._unmap(response)

    @cast_models
    def create_contract_fixed_rate(
        self, request_body: ContractToCreateContainerOngoingTimeBased
    ) -> ContractContainerOngoingTimeBased:
        """Create a new Deel contract.

        :param request_body: The request body.
        :type request_body: ContractToCreateContainerOngoingTimeBased
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractContainerOngoingTimeBased
        """

        Validator(ContractToCreateContainerOngoingTimeBased).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/fixed-rate", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ContractContainerOngoingTimeBased._unmap(response)

    @cast_models
    def create_contract(
        self, request_body: ContractToCreateContainer
    ) -> ContractContainer:
        """Create a new Deel contract.

        :param request_body: The request body.
        :type request_body: ContractToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractContainer
        """

        Validator(ContractToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/contracts", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ContractContainer._unmap(response)

    @cast_models
    def get_contract_preview(self, contract_id: str, template_id: str = None) -> str:
        """Retrieve an IC and EOR contract agreement content in HTML. If no template is specified, the default or currently assigned template will be used. This endpoint does not support Global Payroll contract type.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param template_id: ID of an existing contract template., defaults to None
        :type template_id: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: str
        """

        Validator(str).validate(contract_id)
        Validator(str).is_optional().validate(template_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/preview",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_query("templateId", template_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def amend_contract_details(
        self, request_body: ContractToAmendDetailsContainer, contract_id: str
    ) -> GenericResultCreated:
        """Amend the details of a contract. Please note that if the contract is already signed or active, then the update will have to be approved and re-signed for to take effect.

        :param request_body: The request body.
        :type request_body: ContractToAmendDetailsContainer
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(ContractToAmendDetailsContainer).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/amendments",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def terminate_contract(
        self, request_body: ContractToTerminateContainer, contract_id: str
    ) -> ContractTerminationResultContainer:
        """Terminate an active contract.

        :param request_body: The request body.
        :type request_body: ContractToTerminateContainer
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractTerminationResultContainer
        """

        Validator(ContractToTerminateContainer).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/terminations",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ContractTerminationResultContainer._unmap(response)

    @cast_models
    def add_premium(
        self, contract_id: str, request_body: PremiumToAddContainer = None
    ) -> PremiumResultAddedContainer:
        """Add additional protection against misclassification by upgrading to Deel Premium.

        :param request_body: The request body., defaults to None
        :type request_body: PremiumToAddContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PremiumResultAddedContainer
        """

        Validator(PremiumToAddContainer).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/premium",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return PremiumResultAddedContainer._unmap(response)

    @cast_models
    def remove_premium_from_contract(
        self, contract_id: str, reason: str = None
    ) -> GenericResultDeleted:
        """Remove Deel Premium from an existing contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param reason: Reason for deleting an existing Deel Premium from a contract., defaults to None
        :type reason: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(contract_id)
        Validator(str).is_optional().validate(reason)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/premium",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_query("reason", reason)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class ContractToTerminate(BaseModel):
    """ContractToTerminate

    :param completion_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type completion_date: str, optional
    :param terminate_now: terminate_now, defaults to None
    :type terminate_now: bool, optional
    :param message: A message for the termination, defaults to None
    :type message: str, optional
    """

    def __init__(
        self,
        completion_date: str = None,
        terminate_now: bool = None,
        message: str = None,
    ):
        if completion_date is not None:
            self.completion_date = self._pattern_matching(
                completion_date, "^\d{4}-\d{2}-\d{2}$", "completion_date"
            )
        if terminate_now is not None:
            self.terminate_now = terminate_now
        if message is not None:
            self.message = message



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .basic_contract import BasicContract
from .page_info import PageInfo


@JsonMap({})
class ContractListContainer(BaseModel):
    """ContractListContainer

    :param data: data
    :type data: List[BasicContract]
    :param page: page
    :type page: PageInfo
    """

    def __init__(self, data: List[BasicContract], page: PageInfo):
        self.data = self._define_list(data, BasicContract)
        self.page = self._define_object(page, PageInfo)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class TimesheetStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar APPROVED: "approved"
    :vartype APPROVED: str
    :cvar DECLINED: "declined"
    :vartype DECLINED: str
    :cvar NOT_PAYABLE: "not_payable"
    :vartype NOT_PAYABLE: str
    :cvar PAID: "paid"
    :vartype PAID: str
    :cvar PENDING: "pending"
    :vartype PENDING: str
    :cvar PROCESSING: "processing"
    :vartype PROCESSING: str
    """

    APPROVED = "approved"
    DECLINED = "declined"
    NOT_PAYABLE = "not_payable"
    PAID = "paid"
    PENDING = "pending"
    PROCESSING = "processing"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, TimesheetStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class CalculateFinalPaymentCalculationType(Enum):
    """An enumeration representing different categories.

    :cvar CUSTOM_AMOUNT: "CUSTOM_AMOUNT"
    :vartype CUSTOM_AMOUNT: str
    :cvar WORK_DAYS: "WORK_DAYS"
    :vartype WORK_DAYS: str
    :cvar CALENDAR_DAYS: "CALENDAR_DAYS"
    :vartype CALENDAR_DAYS: str
    :cvar FULL_AMOUNT: "FULL_AMOUNT"
    :vartype FULL_AMOUNT: str
    """

    CUSTOM_AMOUNT = "CUSTOM_AMOUNT"
    WORK_DAYS = "WORK_DAYS"
    CALENDAR_DAYS = "CALENDAR_DAYS"
    FULL_AMOUNT = "FULL_AMOUNT"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                CalculateFinalPaymentCalculationType._member_map_.values(),
            )
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_contract_created import EorContractCreated


@JsonMap({})
class EorContractCreatedContainer(BaseModel):
    """EorContractCreatedContainer

    :param data: data
    :type data: EorContractCreated
    """

    def __init__(self, data: EorContractCreated):
        self.data = self._define_object(data, EorContractCreated)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GlobalPayrollG2NReport(BaseModel):
    """GlobalPayrollG2NReport

    :param net_pay: net_pay, defaults to None
    :type net_pay: str, optional
    :param ee_loans: ee_loans, defaults to None
    :type ee_loans: str, optional
    :param contract_id: contract_id, defaults to None
    :type contract_id: str, optional
    :param annual_bonus: annual_bonus, defaults to None
    :type annual_bonus: float, optional
    :param car_allowance: car_allowance, defaults to None
    :type car_allowance: float, optional
    :param esp_deduction: esp_deduction, defaults to None
    :type esp_deduction: float, optional
    :param employee_name: employee_name, defaults to None
    :type employee_name: str, optional
    :param employer_cost: employer_cost, defaults to None
    :type employer_cost: str, optional
    :param funding_amount: funding_amount, defaults to None
    :type funding_amount: float, optional
    :param home_allowance: home_allowance, defaults to None
    :type home_allowance: str, optional
    :param ee_income_tax: ee_income_tax, defaults to None
    :type ee_income_tax: str, optional
    :param employee_number: employee_number, defaults to None
    :type employee_number: float, optional
    :param backdated_salary: backdated_salary, defaults to None
    :type backdated_salary: str, optional
    :param employer_contribution: employer_contribution, defaults to None
    :type employer_contribution: str, optional
    :param er_occupational_risk: er_occupational_risk, defaults to None
    :type er_occupational_risk: str, optional
    :param employment_status_tag_internal: employment_status_tag_internal, defaults to None
    :type employment_status_tag_internal: str, optional
    :param employee_job_title: employee_job_title, defaults to None
    :type employee_job_title: str, optional
    :param employee_division: employee_division, defaults to None
    :type employee_division: str, optional
    :param employee_department: employee_department, defaults to None
    :type employee_department: str, optional
    :param pension_contribution: pension_contribution, defaults to None
    :type pension_contribution: float, optional
    :param cost_center: cost_center, defaults to None
    :type cost_center: str, optional
    """

    def __init__(
        self,
        net_pay: str = None,
        ee_loans: str = None,
        contract_id: str = None,
        annual_bonus: float = None,
        car_allowance: float = None,
        esp_deduction: float = None,
        employee_name: str = None,
        employer_cost: str = None,
        funding_amount: float = None,
        home_allowance: str = None,
        ee_income_tax: str = None,
        employee_number: float = None,
        backdated_salary: str = None,
        employer_contribution: str = None,
        er_occupational_risk: str = None,
        employment_status_tag_internal: str = None,
        employee_job_title: str = None,
        employee_division: str = None,
        employee_department: str = None,
        pension_contribution: float = None,
        cost_center: str = None,
    ):
        if net_pay is not None:
            self.net_pay = net_pay
        if ee_loans is not None:
            self.ee_loans = ee_loans
        if contract_id is not None:
            self.contract_id = contract_id
        if annual_bonus is not None:
            self.annual_bonus = annual_bonus
        if car_allowance is not None:
            self.car_allowance = car_allowance
        if esp_deduction is not None:
            self.esp_deduction = esp_deduction
        if employee_name is not None:
            self.employee_name = employee_name
        if employer_cost is not None:
            self.employer_cost = employer_cost
        if funding_amount is not None:
            self.funding_amount = funding_amount
        if home_allowance is not None:
            self.home_allowance = home_allowance
        if ee_income_tax is not None:
            self.ee_income_tax = ee_income_tax
        if employee_number is not None:
            self.employee_number = employee_number
        if backdated_salary is not None:
            self.backdated_salary = backdated_salary
        if employer_contribution is not None:
            self.employer_contribution = employer_contribution
        if er_occupational_risk is not None:
            self.er_occupational_risk = er_occupational_risk
        if employment_status_tag_internal is not None:
            self.employment_status_tag_internal = employment_status_tag_internal
        if employee_job_title is not None:
            self.employee_job_title = employee_job_title
        if employee_division is not None:
            self.employee_division = employee_division
        if employee_department is not None:
            self.employee_department = employee_department
        if pension_contribution is not None:
            self.pension_contribution = pension_contribution
        if cost_center is not None:
            self.cost_center = cost_center



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .timesheet_to_update import TimesheetToUpdate


@JsonMap({})
class TimesheetToUpdateContainer(BaseModel):
    """TimesheetToUpdateContainer

    :param data: Details of the timesheet to update. Both a client or a contractor may update a timesheet.
    :type data: TimesheetToUpdate
    """

    def __init__(self, data: TimesheetToUpdate):
        self.data = self._define_object(data, TimesheetToUpdate)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .file_attachment_info import FileAttachmentInfo


@JsonMap({})
class ContractDocumentContainer(BaseModel):
    """ContractDocumentContainer

    :param data: This object is used for linking file attachments to your records.
    :type data: FileAttachmentInfo
    """

    def __init__(self, data: FileAttachmentInfo):
        self.data = self._define_object(data, FileAttachmentInfo)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class TimeoffTypeListContainer(BaseModel):
    """TimeoffTypeListContainer

    :param data: data
    :type data: List[str]
    """

    def __init__(self, data: List[str]):
        self.data = data



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class TimesheetApprover(BaseModel):
    """TimesheetApprover

    :param full_name: full_name
    :type full_name: str
    :param email: email
    :type email: str
    :param approved: approved
    :type approved: bool
    """

    def __init__(self, full_name: str, email: str, approved: bool):
        self.full_name = full_name
        self.email = email
        self.approved = approved



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.worker_termination_container import WorkerTerminationContainer
from ..models.worker_termination_body_container import WorkerTerminationBodyContainer
from ..models.worker_bank_accounts_info_container import WorkerBankAccountsInfoContainer
from ..models.utils.cast_models import cast_models
from ..models.gp_payslips_list_container import GpPayslipsListContainer
from ..models.gp_payslip_download_url_container import GpPayslipDownloadUrlContainer
from ..models.gp_payroll_event_report_container import GpPayrollEventReportContainer
from ..models.gp_employee_pto_to_update_container import GpEmployeePtoToUpdateContainer
from ..models.gp_employee_information_updated_container import (
    GpEmployeeInformationUpdatedContainer,
)
from ..models.gp_employee_information_to_update_container import (
    GpEmployeeInformationToUpdateContainer,
)
from ..models.gp_employee_compensation_updated_container import (
    GpEmployeeCompensationUpdatedContainer,
)
from ..models.gp_employee_compensation_to_update_container import (
    GpEmployeeCompensationToUpdateContainer,
)
from ..models.gp_employee_address_updated_container import (
    GpEmployeeAddressUpdatedContainer,
)
from ..models.gp_employee_address_to_update_container import (
    GpEmployeeAddressToUpdateContainer,
)
from ..models.gp_contract_to_create_container import GpContractToCreateContainer
from ..models.gp_contract_created_container import GpContractCreatedContainer
from ..models.global_payroll_g2_n_report_container import (
    GlobalPayrollG2NReportContainer,
)
from ..models.generic_result_updated import GenericResultUpdated
from ..models.bank_account_updated_container import BankAccountUpdatedContainer
from ..models.bank_account_guide_container import BankAccountGuideContainer
from ..models.add_worker_bank_account_container import AddWorkerBankAccountContainer


class GlobalPayrollService(BaseService):

    @cast_models
    def create_gp_contract(
        self, request_body: GpContractToCreateContainer
    ) -> GpContractCreatedContainer:
        """Create a Global Payroll contract.

        :param request_body: The request body.
        :type request_body: GpContractToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpContractCreatedContainer
        """

        Validator(GpContractToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/contracts/gp", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GpContractCreatedContainer._unmap(response)

    @cast_models
    def get_worker_payslips(self, worker_id: str) -> GpPayslipsListContainer:
        """Get of payslips for an employee.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpPayslipsListContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/payslips",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GpPayslipsListContainer._unmap(response)

    @cast_models
    def update_gp_employee_address(
        self, request_body: GpEmployeeAddressToUpdateContainer, worker_id: str
    ) -> GpEmployeeAddressUpdatedContainer:
        """Update the address of a Global Payroll employee.

        :param request_body: The request body.
        :type request_body: GpEmployeeAddressToUpdateContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpEmployeeAddressUpdatedContainer
        """

        Validator(GpEmployeeAddressToUpdateContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/address",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GpEmployeeAddressUpdatedContainer._unmap(response)

    @cast_models
    def get_gp_bank_accounts(self, worker_id: str) -> WorkerBankAccountsInfoContainer:
        """Retrieve all bank accounts for an employee.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: WorkerBankAccountsInfoContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/banks",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return WorkerBankAccountsInfoContainer._unmap(response)

    @cast_models
    def add_gp_bank_account(
        self, request_body: AddWorkerBankAccountContainer, worker_id: str
    ) -> BankAccountUpdatedContainer:
        """Add a new bank account for an employee.

        :param request_body: The request body.
        :type request_body: AddWorkerBankAccountContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: BankAccountUpdatedContainer
        """

        Validator(AddWorkerBankAccountContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/banks",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return BankAccountUpdatedContainer._unmap(response)

    @cast_models
    def patch_gp_bank_account(
        self, request_body: AddWorkerBankAccountContainer, worker_id: str, bank_id: str
    ) -> BankAccountUpdatedContainer:
        """Modify bank account for an employee.

        :param request_body: The request body.
        :type request_body: AddWorkerBankAccountContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        :param bank_id: Bank Account id.
        :type bank_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: BankAccountUpdatedContainer
        """

        Validator(AddWorkerBankAccountContainer).validate(request_body)
        Validator(str).validate(worker_id)
        Validator(str).validate(bank_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/banks/{{bank_id}}",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .add_path("bank_id", bank_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return BankAccountUpdatedContainer._unmap(response)

    @cast_models
    def get_gp_bank_guide(self, worker_id: str) -> BankAccountGuideContainer:
        """Retrieve the bank form guide for employee.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: BankAccountGuideContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/banks/guide",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return BankAccountGuideContainer._unmap(response)

    @cast_models
    def update_gp_employee_compensation(
        self, request_body: GpEmployeeCompensationToUpdateContainer, worker_id: str
    ) -> GpEmployeeCompensationUpdatedContainer:
        """Update the compensation of a Global Payroll employee. Returns the full compensation history including the update.

        :param request_body: The request body.
        :type request_body: GpEmployeeCompensationToUpdateContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpEmployeeCompensationUpdatedContainer
        """

        Validator(GpEmployeeCompensationToUpdateContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/compensation",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GpEmployeeCompensationUpdatedContainer._unmap(response)

    @cast_models
    def update_gp_employee_pto(
        self, request_body: GpEmployeePtoToUpdateContainer, worker_id: str
    ) -> GenericResultUpdated:
        """Update the PTO policy of a Global Payroll employee.

        :param request_body: The request body.
        :type request_body: GpEmployeePtoToUpdateContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(GpEmployeePtoToUpdateContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/pto-policy",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def update_gp_employee_information(
        self, request_body: GpEmployeeInformationToUpdateContainer, worker_id: str
    ) -> GpEmployeeInformationUpdatedContainer:
        """Update Global Payroll employee information.

        :param request_body: The request body.
        :type request_body: GpEmployeeInformationToUpdateContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpEmployeeInformationUpdatedContainer
        """

        Validator(GpEmployeeInformationToUpdateContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/employee-information",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GpEmployeeInformationUpdatedContainer._unmap(response)

    @cast_models
    def get_download_url_for_gp_payslip(
        self, worker_id: str, payslip_id: str
    ) -> GpPayslipDownloadUrlContainer:
        """Get download url for GP payslip.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        :param payslip_id: Unique identifier for a payslip.
        :type payslip_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpPayslipDownloadUrlContainer
        """

        Validator(str).validate(worker_id)
        Validator(str).validate(payslip_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/payslips/{{payslip_id}}/download",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .add_path("payslip_id", payslip_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GpPayslipDownloadUrlContainer._unmap(response)

    @cast_models
    def get_gp_legal_entities(
        self, legal_entity_id: str, start_date: str = None
    ) -> GpPayrollEventReportContainer:
        """Get list of global payroll events by legal entities.

        :param legal_entity_id: Id of an legal entity.
        :type legal_entity_id: str
        :param start_date: Start date of payroll report., defaults to None
        :type start_date: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GpPayrollEventReportContainer
        """

        Validator(str).validate(legal_entity_id)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(start_date)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/legal-entities/{{legal_entity_id}}/reports",
                self.get_default_headers(),
            )
            .add_path("legal_entity_id", legal_entity_id)
            .add_query("start_date", start_date, nullable=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GpPayrollEventReportContainer._unmap(response)

    @cast_models
    def get_gross_to_net_gp_reports(
        self, gp_report_id: str
    ) -> GlobalPayrollG2NReportContainer:
        """Get list of global payroll reports detailing gross-to-net calculations.

        :param gp_report_id: Id of gp report.
        :type gp_report_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GlobalPayrollG2NReportContainer
        """

        Validator(str).validate(gp_report_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/reports/{{gp_report_id}}/gross_to_net",
                self.get_default_headers(),
            )
            .add_path("gp_report_id", gp_report_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GlobalPayrollG2NReportContainer._unmap(response)

    @cast_models
    def download_gross_to_net_gp_report(self, gp_report_id: str) -> str:
        """Download global payroll reports detailing gross-to-net calculations.

        :param gp_report_id: Id of gp report.
        :type gp_report_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: str
        """

        Validator(str).validate(gp_report_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/reports/{{gp_report_id}}/gross_to_net/csv",
                self.get_default_headers(),
            )
            .add_path("gp_report_id", gp_report_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def request_termination(
        self, request_body: WorkerTerminationBodyContainer, worker_id: str
    ) -> WorkerTerminationContainer:
        """Request a termination for a global payroll employee. A successful call starts the termination process and does not confirm termination.

        :param request_body: The request body.
        :type request_body: WorkerTerminationBodyContainer
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: WorkerTerminationContainer
        """

        Validator(WorkerTerminationBodyContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/gp/workers/{{worker_id}}/terminations",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return WorkerTerminationContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .file_ref_type_enum import FileRefTypeEnum


@JsonMap({})
class InputToCreateFileRefData(BaseModel):
    """InputToCreateFileRefData

    :param content_type: type of files allowed when uploading attachments
    :type content_type: FileRefTypeEnum
    """

    def __init__(self, content_type: FileRefTypeEnum):
        self.content_type = self._enum_matching(
            content_type, FileRefTypeEnum.list(), "content_type"
        )


@JsonMap({})
class InputToCreateFileRef(BaseModel):
    """InputToCreateFileRef

    :param data: data
    :type data: InputToCreateFileRefData
    """

    def __init__(self, data: InputToCreateFileRefData):
        self.data = self._define_object(data, InputToCreateFileRefData)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .worker_document import WorkerDocument


@JsonMap({})
class WorkerDocumentsByIdContainer(BaseModel):
    """WorkerDocumentsByIdContainer

    :param data: List of worker documents.
    :type data: List[WorkerDocument]
    """

    def __init__(self, data: List[WorkerDocument]):
        self.data = self._define_list(data, WorkerDocument)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GenericResultUpdatedData(BaseModel):
    """GenericResultUpdatedData

    :param updated: updated
    :type updated: bool
    """

    def __init__(self, updated: bool):
        self.updated = updated


@JsonMap({})
class GenericResultUpdated(BaseModel):
    """GenericResultUpdated

    :param data: data
    :type data: GenericResultUpdatedData
    """

    def __init__(self, data: GenericResultUpdatedData):
        self.data = self._define_object(data, GenericResultUpdatedData)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class TimeoffsTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar VACATION: "VACATION"
    :vartype VACATION: str
    :cvar SICK_LEAVE: "SICK_LEAVE"
    :vartype SICK_LEAVE: str
    :cvar OTHER: "OTHER"
    :vartype OTHER: str
    """

    VACATION = "VACATION"
    SICK_LEAVE = "SICK_LEAVE"
    OTHER = "OTHER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, TimeoffsTypeEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class SalaryFrequencyScaleEnum(Enum):
    """An enumeration representing different categories.

    :cvar MONTHLY: "monthly"
    :vartype MONTHLY: str
    :cvar QUARTERLY: "quarterly"
    :vartype QUARTERLY: str
    :cvar ANNUALLY: "annually"
    :vartype ANNUALLY: str
    :cvar SPECIFIC_MONTH: "specific_month"
    :vartype SPECIFIC_MONTH: str
    :cvar SPECIFIC_DATE: "specific_date"
    :vartype SPECIFIC_DATE: str
    """

    MONTHLY = "monthly"
    QUARTERLY = "quarterly"
    ANNUALLY = "annually"
    SPECIFIC_MONTH = "specific_month"
    SPECIFIC_DATE = "specific_date"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, SalaryFrequencyScaleEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.task_list_container import TaskListContainer
from ..models.task_created_container import TaskCreatedContainer
from ..models.request_body_to_create_pgo_task_reviews_reviews_container import (
    RequestBodyToCreatePgoTaskReviewsReviewsContainer,
)
from ..models.request_body_to_create_pgo_task_reviews_by_id_reviews_container import (
    RequestBodyToCreatePgoTaskReviewsByIdReviewsContainer,
)
from ..models.input_to_create_pgo_task import InputToCreatePgoTask
from ..models.generic_result_deleted import GenericResultDeleted


class TasksService(BaseService):

    @cast_models
    def get_tasks_by_contract(self, contract_id: str) -> TaskListContainer:
        """Retrieve a list of tasks for a given contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TaskListContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/tasks",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return TaskListContainer._unmap(response)

    @cast_models
    def create_contract_pgo_tak(
        self, request_body: InputToCreatePgoTask, contract_id: str
    ) -> TaskCreatedContainer:
        """Create a new task for the contractor.

        :param request_body: The request body.
        :type request_body: InputToCreatePgoTask
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TaskCreatedContainer
        """

        Validator(InputToCreatePgoTask).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/tasks",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return TaskCreatedContainer._unmap(response)

    @cast_models
    def create_task_many_review(
        self,
        contract_id: str,
        request_body: RequestBodyToCreatePgoTaskReviewsReviewsContainer = None,
    ) -> TaskListContainer:
        """Review multiple tasks to approve or decline the submitted work.

        :param request_body: The request body., defaults to None
        :type request_body: RequestBodyToCreatePgoTaskReviewsReviewsContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TaskListContainer
        """

        Validator(
            RequestBodyToCreatePgoTaskReviewsReviewsContainer
        ).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/tasks/many/reviews",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return TaskListContainer._unmap(response)

    @cast_models
    def create_task_review_by_id(
        self,
        contract_id: str,
        task_id: str,
        request_body: RequestBodyToCreatePgoTaskReviewsByIdReviewsContainer = None,
    ) -> TaskListContainer:
        """Review a single task to approve or decline the submitted work.

        :param request_body: The request body., defaults to None
        :type request_body: RequestBodyToCreatePgoTaskReviewsByIdReviewsContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        :param task_id: ID of task entity.
        :type task_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TaskListContainer
        """

        Validator(
            RequestBodyToCreatePgoTaskReviewsByIdReviewsContainer
        ).is_optional().validate(request_body)
        Validator(str).validate(contract_id)
        Validator(str).validate(task_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/tasks/{{task_id}}/reviews",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("task_id", task_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return TaskListContainer._unmap(response)

    @cast_models
    def delete_contract_pgo_tak(
        self, contract_id: str, task_id: str, reason: str = None
    ) -> GenericResultDeleted:
        """Delete task from the contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param task_id: ID of task entity.
        :type task_id: str
        :param reason: Reason for deleting an existing task, defaults to None
        :type reason: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(contract_id)
        Validator(str).validate(task_id)
        Validator(str).is_optional().validate(reason)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/tasks/{{task_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("task_id", task_id)
            .add_query("reason", reason)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class CandidateStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar OFFER_ACCEPTED: "offer-accepted"
    :vartype OFFER_ACCEPTED: str
    :cvar OFFER_SENT: "offer-sent"
    :vartype OFFER_SENT: str
    :cvar OFFER_DECLINED: "offer-declined"
    :vartype OFFER_DECLINED: str
    :cvar OFFER_DELETED: "offer-deleted"
    :vartype OFFER_DELETED: str
    """

    OFFER_ACCEPTED = "offer-accepted"
    OFFER_SENT = "offer-sent"
    OFFER_DECLINED = "offer-declined"
    OFFER_DELETED = "offer-deleted"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, CandidateStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .estimate_first_payment import EstimateFirstPayment


@JsonMap({})
class EstimateFirstPaymentContainer(BaseModel):
    """EstimateFirstPaymentContainer

    :param data: Details of pro rata payment to create.
    :type data: EstimateFirstPayment
    """

    def __init__(self, data: EstimateFirstPayment):
        self.data = self._define_object(data, EstimateFirstPayment)



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.timeoff_to_review_container import TimeoffToReviewContainer
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.eor_timeoffs_item_container import EorTimeoffsItemContainer
from ..models.eor_timeoffs_container import EorTimeoffsContainer
from ..models.eor_entitlements_container import EorEntitlementsContainer
from ..models.eor_client_timeoffs_container import EorClientTimeoffsContainer
from ..models.create_timeoff_container import CreateTimeoffContainer


class TimeOffService(BaseService):

    @cast_models
    def get_eor_time_offs(self, contract_id: str) -> EorTimeoffsContainer:
        """Retrieve the list of time off requests by an employee.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorTimeoffsContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/time-offs",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorTimeoffsContainer._unmap(response)

    @cast_models
    def create_eor_time_offs(
        self, contract_id: str, request_body: CreateTimeoffContainer = None
    ) -> EorTimeoffsItemContainer:
        """Add a time off request for a full-time employee.

        :param request_body: The request body., defaults to None
        :type request_body: CreateTimeoffContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorTimeoffsItemContainer
        """

        Validator(CreateTimeoffContainer).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/time-offs",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return EorTimeoffsItemContainer._unmap(response)

    @cast_models
    def edit_eor_time_offs(
        self,
        contract_id: str,
        timeoff_id: str,
        request_body: CreateTimeoffContainer = None,
    ) -> EorTimeoffsItemContainer:
        """Edit a time off request for a full-time employee.

        :param request_body: The request body., defaults to None
        :type request_body: CreateTimeoffContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        :param timeoff_id: Time off Id
        :type timeoff_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorTimeoffsItemContainer
        """

        Validator(CreateTimeoffContainer).is_optional().validate(request_body)
        Validator(str).validate(contract_id)
        Validator(str).validate(timeoff_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/time-offs/{{timeoff_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("timeoff_id", timeoff_id)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return EorTimeoffsItemContainer._unmap(response)

    @cast_models
    def delete_eor_time_offs(
        self, contract_id: str, timeoff_id: str
    ) -> GenericResultDeleted:
        """Cancel a time off request for an employee.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param timeoff_id: Time off Id
        :type timeoff_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(contract_id)
        Validator(str).validate(timeoff_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/time-offs/{{timeoff_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("timeoff_id", timeoff_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def get_eor_entitlements(self, contract_id: str) -> EorEntitlementsContainer:
        """Retrieve a list of time off entitlements for a full-time employee.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorEntitlementsContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/entitlements",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorEntitlementsContainer._unmap(response)

    @cast_models
    def get_eor_client_time_offs(self) -> EorClientTimeoffsContainer:
        """List of time offs for all employees in your organization.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EorClientTimeoffsContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/time-offs", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorClientTimeoffsContainer._unmap(response)

    @cast_models
    def review_timeoff(
        self, request_body: TimeoffToReviewContainer, timeoff_id: str
    ) -> GenericResultUpdated:
        """Approve or decline an employee's time off request.

        :param request_body: The request body.
        :type request_body: TimeoffToReviewContainer
        :param timeoff_id: Time off Id
        :type timeoff_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(TimeoffToReviewContainer).validate(request_body)
        Validator(str).validate(timeoff_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/time-offs/{{timeoff_id}}/review",
                self.get_default_headers(),
            )
            .add_path("timeoff_id", timeoff_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)
